Package: SANTA
Type: Package
Title: Spatial Analysis of Network Associations
Version: 2.49.0
Authors@R: person("Alex", "Cornish", role = c("cre","aut"),
             email = "alexcornish88@gmail.com")
Date: 17 June 2024
Imports: graphics, Matrix, methods, stats
Depends: R (>= 4.1), igraph
Suggests: BiocGenerics, BioNet, formatR, knitr, msm, org.Sc.sgd.db,
        markdown, rmarkdown, RUnit
Description: This package provides methods for measuring the strength of association between a network and a phenotype. It does this by measuring clustering of the phenotype across the network (Knet). Vertices can also be individually ranked by their strength of association with high-weight vertices (Knode).
License: GPL (>= 2)
LazyLoad: yes
VignetteBuilder: knitr
biocViews: Network, NetworkEnrichment, Clustering
BugReports: https://github.com/alexjcornish/SANTA
git_url: https://git.bioconductor.org/packages/SANTA
git_branch: devel
git_last_commit: 09482a2
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 00:49:43 UTC; biocbuild
Author: Alex Cornish [cre, aut]
Maintainer: Alex Cornish <alexcornish88@gmail.com>

import(igraph, Matrix)
importFrom("graphics", "abline", "hist", "lines", "par", "polygon")
importFrom("methods", "is")
importFrom("stats", "pnorm", "quantile", "runif", "sd")

export(
	BinGraph,
	CreateGrid, 
	Compactness,
	DistGraph, 
	GraphDiffusion, 
	GraphMFPT,
	Knet, 
	Knode, 
	plot.Knet, 
	SpreadHits
)

S3method(plot, Knet)

useDynLib(SANTA)

# Santa 🎅

**A modern, high-performance package manager meta-tool that works across platforms.**

Santa helps you install and manage packages across multiple platforms and package managers with a single command. Whether you're switching between macOS, Linux, or Windows, Santa ensures your essential tools are always available.

[![Build Status](https://github.com/tylerbutler/santa/workflows/CI/badge.svg)](https://github.com/tylerbutler/santa/actions)
[![codecov](https://codecov.io/gh/tylerbutler/santa/branch/main/graph/badge.svg)](https://codecov.io/gh/tylerbutler/santa)
[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)

## Why Santa?

### You regularly use tools not installed by default

You're a modern developer. You prefer `ripgrep` over `grep`, `bat` over `cat`, and `fd` over `find`. The problem? They're not installed by default. You're stuck installing them manually with whatever package manager is available.

Santa gives you **one command** to install your entire "standard developer toolkit."

### You work across different operating systems

Isn't it annoying when you log into a new machine and it doesn't have your preferred tools? Or when your tool isn't installable using `apt`, but you don't remember which package manager _does_ have it?

Santa simplifies this workflow. Santa knows where your packages can be installed from and will install them from the best available source.

## Features

### 🚀 Performance
- **67-90% faster** concurrent package operations via async subprocess execution
- **Professional-grade caching** with TTL and LRU eviction
- **Memory efficient** with zero unnecessary allocations

### 🔧 Cross-Platform
- **Linux, macOS, and Windows** support
- **Multiple package managers**: Homebrew, Cargo, APT, Pacman, AUR, Scoop, Nix
- **Automatic fallback** to available package managers

### 🌟 User-Friendly
- **Configuration hot-reloading** - changes take effect immediately
- **Shell completions** for bash, zsh, and fish
- **Environment variable overrides** for all settings
- **CCL configuration format** - simple and readable

### 🛡️ Reliable
- **Comprehensive error handling** with helpful messages
- **Security hardening** with input sanitization
- **144 comprehensive tests** with >90% code coverage

## Installation

### From Releases (Recommended)

Download pre-built binaries from the [releases page](https://github.com/tylerbutler/santa/releases):

```bash
# Linux (x64)
curl -L https://github.com/tylerbutler/santa/releases/latest/download/santa-linux-x64.tar.gz | tar xz

# macOS (Apple Silicon)
curl -L https://github.com/tylerbutler/santa/releases/latest/download/santa-macos-aarch64.tar.gz | tar xz

# Windows (PowerShell)
irm https://github.com/tylerbutler/santa/releases/latest/download/santa-windows-x64.zip -OutFile santa.zip
Expand-Archive santa.zip
```

### From Source

```bash
cargo install santa-cli
```

### Verify Installation

```bash
santa --version
```

## Quick Start

### 1. Check Status

See what packages are available and their installation status:

```bash
santa status
```

### 2. Install Packages

Install all configured packages:

```bash
santa install
```

Install specific packages:

```bash
santa install ripgrep bat fd
```

### 3. View Configuration

```bash
santa config
```

### 4. Set Up Shell Completions

```bash
# Bash
santa completions bash >> ~/.bashrc

# Zsh
santa completions zsh >> ~/.zshrc

# Fish
santa completions fish > ~/.config/fish/completions/santa.fish
```

## Configuration

Santa uses a CCL (Categorical Configuration Language) configuration file.

**Default location:** `~/.config/santa/config.ccl`

### Basic Configuration

```ccl
/= Package sources in order of preference
sources =
  = brew
  = cargo
  = apt
  = pacman
  = scoop
  = nix

/= Packages to install
packages =
  = bat
  = ripgrep
  = fd
  = exa
  = bottom
  = git-delta
  = chezmoi
```

### Supported Package Managers

| Package Manager | Platforms | Status |
|----------------|-----------|--------|
| **Homebrew** | macOS, Linux | ✅ Full Support |
| **Cargo** | All | ✅ Full Support |
| **APT** | Debian, Ubuntu | ✅ Full Support |
| **Pacman** | Arch Linux | ✅ Full Support |
| **AUR** | Arch Linux | ✅ Full Support |
| **Scoop** | Windows | ✅ Full Support |
| **Nix** | All | ✅ Full Support |

### Environment Variables

Override any configuration setting with environment variables:

```bash
export SANTA_SOURCES="brew,cargo,apt"
export SANTA_PACKAGES="git,rust,ripgrep"
export SANTA_LOG_LEVEL="debug"
export SANTA_BUILTIN_ONLY="true"
```

**Available Variables:**

- `SANTA_LOG_LEVEL` - Set log level (trace, debug, info, warn, error)
- `SANTA_CONFIG_PATH` - Override configuration file path
- `SANTA_SOURCES` - Override package sources (comma-separated)
- `SANTA_PACKAGES` - Override package list (comma-separated)
- `SANTA_BUILTIN_ONLY` - Use builtin configuration only (true/false)
- `SANTA_CACHE_TTL_SECONDS` - Set package cache TTL
- `SANTA_CACHE_SIZE` - Set maximum cache entries
- `SANTA_VERBOSE` - Set verbose logging level (0-3)
- `SANTA_HOT_RELOAD` - Enable configuration hot-reloading (true/false)

## Commands

### `santa status`

Show all packages and their installation status across available package managers.

```bash
santa status

# Show only installed packages
santa status --installed

# Show only missing packages
santa status --missing
```

### `santa install`

Install packages from the configured sources.

```bash
# Install all configured packages
santa install

# Install specific packages
santa install ripgrep bat fd

# Dry run (show what would be installed)
santa install --dry-run
```

### `santa config`

Display current configuration including sources, packages, and settings.

```bash
santa config

# Show configuration file path
santa config --path

# Show resolved configuration (after environment variables)
santa config --resolved
```

### `santa completions`

Generate shell completion scripts.

```bash
# Generate for bash
santa completions bash

# Generate for zsh
santa completions zsh

# Generate for fish
santa completions fish
```

## Advanced Usage

### Custom Configuration File

```bash
santa --config ~/my-config.ccl status
```

### Verbose Output

```bash
# Level 1: Basic info
santa -v install

# Level 2: Detailed info
santa -vv install

# Level 3: Debug info
santa -vvv install
```

### Using with CI/CD

```bash
# Set packages via environment variable
export SANTA_PACKAGES="jq,yq,gh"
santa install

# Use builtin configuration only (no config file)
export SANTA_BUILTIN_ONLY=true
santa install
```

## Troubleshooting

### Package Not Found

If a package isn't found, check:
1. The package name is correct for your package manager
2. Your package manager is in the `sources` list
3. Your package manager is installed and in your PATH

```bash
# Check which package managers are available
santa config
```

### Permission Errors

Some package managers require elevated privileges:

```bash
# Linux/macOS
sudo santa install

# Windows (run PowerShell as Administrator)
santa install
```

### Configuration Not Loading

Verify your configuration file path:

```bash
santa config --path
```

Or specify the path explicitly:

```bash
santa --config ~/.config/santa/config.ccl status
```

## Performance

Santa is designed for speed:

- **67-90% faster** than sequential package operations
- **Parallel execution** of package manager queries
- **Intelligent caching** to avoid redundant operations
- **Async I/O** for non-blocking operations

## Contributing

Contributions are welcome! Please see the [main repository](https://github.com/tylerbutler/santa) for development guidelines.

## License

Licensed under the [MIT License](LICENSE).

---

**Made with ❤️ for developers who work across platforms**

Knet <- function(
    g, 
    nperm=100, 
    dist.method=c("shortest.paths", "diffusion", "mfpt"), 
    vertex.attr="pheno",
    edge.attr=NULL,
    correct.factor=1,
    nsteps=1000, 
    prob=c(0, 0.05, 0.5, 0.95, 1),
    B=NULL,
    verbose=TRUE
){
    # calculate the Knet function for a graph, along with permutations if required
    
    # check that vertex weights and edge distances are present and suitable. Convert if neccessary
    dist.method <- match.arg(dist.method)
    g <- CheckAttributes(g, vertex.attr, edge.attr, verbose)
    
    # the results are saved in a list with an entry for each vertex.attr. Done even if only 1 vertex.attr supplied
    tmp <- vector("list", 6)
    names(tmp) <- c("K.obs", "AUK.obs", "K.perm", "AUK.perm", "K.quan", "pval")
    class(tmp) <- "Knet"
    res <- rep(list(tmp), length(vertex.attr)) 
    names(res) <- vertex.attr
    nvertices <- as.integer(vcount(g))
    
    if (is.null(B)) {
        # compute B and D
        D <- DistGraph(g, edge.attr=edge.attr, dist.method=dist.method, correct.inf=TRUE, correct.factor=correct.factor, verbose=verbose) 
        B <- BinGraph(D, nsteps=nsteps, verbose=verbose) 
        rm(D)
    } else {
        # check that B is of the correct dimensions
        if (!identical(dim(B), rep(nvertices, 2))) stop("B is not of the correct dimensions")
    }
    
    # convert B to a vector and compute the maximum of B
    Bv <- as.integer(as.vector(B))
    maxB <- as.integer(max(B))
    rm(B)
    
    # run the function for each vertex attribute in vertex.attr
    for (attr in vertex.attr) {
        attr.message <- if (length(vertex.attr) == 1) NULL else paste("(", which(vertex.attr == attr), "/", length(vertex.attr), ") ", sep="")
        if (verbose) message("computing the clustering of the '", attr, "' weights ", attr.message, "using ", nperm, " permutations... ", appendLF=FALSE)
        
        # extract vertex weights 
        vertex.weights <- get.vertex.attribute(g, attr)
        vertex.weights.is.na <- is.na(vertex.weights) # don't permute the NA values
        vertex.weights[vertex.weights.is.na] <- 0
        vertex.weights <- as.double(vertex.weights)
                
        # calculate the observed netK and netAUK
        res[[attr]]$K.obs <- .Call("computenetK_fewzeros", Bv, vertex.weights, nvertices, maxB)
        res[[attr]]$AUK.obs <- sum(res[[attr]]$K.obs) / length(res[[attr]]$K.obs)
       
        # if specified, run the Knet function on permutations of the graph
        if (!is.null(nperm) & nperm > 0) {
 	        # run permutations
            res[[attr]]$K.perm <- list()
            for (i in seq_len(nperm)) {
                vertex.weights.shuffled <- Shuffle(vertex.weights, ignore=vertex.weights.is.na)
                res[[attr]]$K.perm[[i]] <- .Call("computenetK_fewzeros", Bv, vertex.weights.shuffled, nvertices, maxB)
            }
            res[[attr]]$K.perm <- do.call("cbind", res[[attr]]$K.perm)
            
            # calculate the quantiles, AUK and p-values (through the z-score) for the permutations
            res[[attr]]$K.quan <- apply(res[[attr]]$K.perm, 1, function(x) quantile(x, prob=prob))
            res[[attr]]$AUK.perm <- apply(res[[attr]]$K.perm, 2, function(x) sum(x) / length(x))
            res[[attr]]$pval <- pnorm((res[[attr]]$AUK.obs - mean(res[[attr]]$AUK.perm)) / sd(res[[attr]]$AUK.perm), lower.tail=FALSE) 
        } else {
            # if no permutations are run, permutation-related statistics are returns equal to NA
            res[[attr]][c("K.perm", "AUK.perm", "K.quan", "pval")] <- NA
        }
         
        if (verbose) message(" done")
    }
    
    # cleanup and output
    if (length(vertex.attr) == 1) res <- res[[1]] # if only one vertex attribute is input, don't return a list of lists of results
    res
}

test_Knet <- function() {
    # setup 
    # graph used:
    # 2 - 1 - 4 - 5
    #     | 
    #     3
    
    g <- graph.empty(5, directed=FALSE)
    g <- add.edges(g, c(1,2,1,3,1,4,4,5))
    g <- set.vertex.attribute(g, "vweights1", value=c(1, 0, 0, 1, 0))
    g <- set.vertex.attribute(g, "vweights2", value=c(0, 1, 0, 0, 1))
    g <- set.vertex.attribute(g, "vweights3", value=c(0.9, 0.1, 0.8, 0.05, 0.15)) # clustering
    g <- set.vertex.attribute(g, "vweights4", value=c(0.1, 0.9, 0.05, 0.15, 0.8)) # no clustering
    g <- set.vertex.attribute(g, "vweights5", value=c(1, 0, 1, NA, NA)) 
    g <- set.edge.attribute(g, "eweights1", value=c(1, 1, 0.01, 1))
    g <- set.edge.attribute(g, "eweights2", value=c(0.01, 0.01, 1, 0.01))
    tol <- 10e-10
    
    # Knet correctly incorperates different binary vertex weights 
    # all edge distances equal 1 as no attribute specified
    suppressMessages(res1 <- Knet(g, nperm=0, vertex.attr="vweights1"))
    suppressMessages(res2 <- Knet(g, nperm=0, vertex.attr="vweights2"))
    checkTrue(tail(res1$K.obs, 1) < tol) # check that the last observed K is basically 0
    checkTrue(tail(res2$K.obs, 1) < tol) # check that the last observed K is basically 0
    checkEquals(res1$AUK.obs, 0.3)
    checkEquals(res2$AUK.obs, 0.1)
   
    # Knet correctly incorperates different continuous vertex weights
    suppressMessages(res3 <- Knet(g, nperm=0, vertex.attr="vweights3"))
    suppressMessages(res4 <- Knet(g, nperm=0, vertex.attr="vweights4"))
    checkTrue(tail(res3$K.obs, 1) < tol) # check that the last observed K is basically 0
    checkTrue(tail(res4$K.obs, 1) < tol) # check that the last observed K is basically 0
    checkEquals(res3$AUK.obs, 0.220625)
    checkEquals(res4$AUK.obs, 0.04875)
     
    # Knet correctly incorperates different edge distances when using the different distance measures
    dist.methods <- c("shortest.paths", "diffusion", "mfpt")
    for (dist.method in dist.methods) {
        suppressMessages(res3 <- Knet(g, nperm=0, dist.method=dist.method, vertex.attr="vweights1", edge.attr="eweights1"))
        suppressMessages(res4 <- Knet(g, nperm=0, dist.method=dist.method, vertex.attr="vweights1", edge.attr="eweights2"))
        checkTrue(res3$AUK.obs > res4$AUK.obs)
    }
    
    # Knet does not shuffle the weights of vertices with weight NA in permutations
    # if this is true, then only AUKs of 0.3 and 0.4 should be seen in the permutations
    # if the weights af these vertices are permuted, then values such as 0.1 would appear
    suppressMessages(res5 <- Knet(g, nperm=100, vertex.attr="vweights5"))
    checkTrue(all(res5$AUK.perm - 0.4 < tol | res5$AUK.perm - 0.3 < tol))
    
    # Knet outputs a list of results containing the correct elements when one vertex attribute is input
    suppressMessages(res6 <- Knet(g, nperm=10, vertex.attr="vweights1", edge.attr="eweights1"))
    checkEquals(names(res6), c("K.obs", "AUK.obs", "K.perm", "AUK.perm", "K.quan", "pval"))
    
    # Knet outputs a list of lists of results containing the correct elements when multiple vertex attributes are input
    vertex.attributes <- c("vweights1", "vweights2")
    suppressMessages(res7 <- Knet(g, nperm=10, vertex.attr=vertex.attributes, edge.attr="eweights1"))
    checkEquals(length(res7), length(vertex.attributes))
    checkEquals(names(res7[[1]]), c("K.obs", "AUK.obs", "K.perm", "AUK.perm", "K.quan", "pval"))
    
    # Knet outputs a list of results, some of which equal NA, when one attribute and no permutations are completed
    suppressMessages(res8 <- Knet(g, nperm=0, vertex.attr="vweights1", edge.attr="eweights1"))
    checkTrue(all(is.na(res8[c("pval", "K.perm", "AUK.perm", "K.quan")])))
}

GraphMFPT <- function(
    g,
    v=V(g),
    edge.attr.weight=NULL,
    average.distances=TRUE
) {
    # calculate the mean first-passage time (MFPT) between every vertex pair
    # Algorithms for Estimating Relative Importance in Networks, White & Smyth, 2003
    # this function will return Inf distances if the graph is not connected
    # any edge attribute specified should be the weight of the edge (higher weights -> more significant) not the distance
    
    if (!is(v, "igraph.vs")) v <- AsiGraph(v, g) # if v is not an igraph-class object, convert
        
    # obtain the sparse unamed adjacency matrix
    adj <- get.adjacency(g, attr=edge.attr.weight, names=FALSE, sparse=TRUE, type="both")
    
    # compute the distance matrix for each of the connected clusters of vertices
    D <- array(Inf, dim=rep(vcount(g), 2))
    c <- clusters(g) 
    for (i in seq_len(c$no)) {
        indices <- which(c$membership == i)
        D[indices, indices] <- MFPTfct(adj[indices, indices])
    }
     
    if (average.distances) D <- (D + t(D)) / 2 # if required, take the average of the reciprical distances
    diag(D) <- 0 # the distance between vertex A and A should always be 0
    
    # return rows for each vertex in v
    D <- as.matrix(D[v, ])
    dimnames(D) <- list(V(g)$name[as.numeric(v)], V(g)$name) 
    
    D
}



MFPTfct <- function(
    adj
) {
    # calculate the mean first-passage time (MFPT) for a fully connected graph from the adjacency matrix
    # note: this function is unable to deal with graphs that are not fully connected
    
    # if the adjacency matrix contains only a single gene, return a 1x1 matrix containing 0
    if (is.null(dim(adj))) return(matrix(0, 1, 1))
    ngenes <- nrow(adj)
    
    A <- adj / apply(adj, 1, sum)  # A: the transition probability matrix (there is always movement)
    I <- Diagonal(x=rep(as.integer(1), ngenes))
    pi <- as.numeric(rep(1/ngenes, ngenes) %*% solve(I - A + 1 / ngenes)) # pi: the stationary distribution of the transition matrix
    Z <- solve(t(t(I - A) - pi))
    as.matrix(t(t(I - Z) + Z[cbind(seq_len(nrow(Z)), seq_len(nrow(Z)))]) %*% (I * (1 / pi))) # M: the mean first passage matrix
}

CheckAttributes <- function(
    g, 
    vertex.attr="pheno", 
    edge.attr=NULL,
    verbose=TRUE
) {
    # check that vertex and edge weights are present in a graph and convert them if neccessary
    
    # check that all supplied vertex attributes are present on the graph
    if (!all(vertex.attr %in% list.vertex.attributes(g))) stop("not all vertex attributes found on graph")
    
    for (attr in vertex.attr) {
        # complete checks on vertex attribute(s)
        vertex.weights <- get.vertex.attribute(g, attr)
        
        # identify the vertex weights that are missing
        missing <- is.na(vertex.weights)
        
        # check that none of the vertex weights are infinite. If there are any, return error
        if (sum(vertex.weights[!missing] == Inf) > 0) stop(attr, " vertex weights contain infinite values")
        
        # check that vertex weights are numeric. If not try to convert. If conversion is not possible, return error
        if (!is.numeric(vertex.weights)) {
            if (is.logical(vertex.weights)) {
                missing.char <- is.na(vertex.weights)
                vertex.weights[missing.char] <- FALSE
            } else {
                warning("non-numeric/non-logical vertex weights found and converted to numerals")
                missing.char <- vertex.weights == "NA"
                vertex.weights[missing.char] <- "0"     	
            }
            
            vertex.weights <- as.numeric(vertex.weights)
            if (sum(is.na(vertex.weights)) > 0) stop("unable to convert non-numeric vertex weights to numerals")
            
            vertex.weights[missing.char] <- NA
            g <- remove.vertex.attribute(g, attr)
            g <- set.vertex.attribute(g, attr, value=vertex.weights)
        }
        
        # check that all vertex weights are greater or equal to 0. If not, return error
        if (!all(vertex.weights[!is.na(vertex.weights)] >= 0)) stop(attr, " vertex weights contain negative values")
    }
    
    if (!is.null(edge.attr)) {
        # complete checks on edge attribute if present
        
        # check that edge distances are present, if not then they are all set equal to 1. This is not an error, as some graphs do not have edge distances
        if (is.null(get.edge.attribute(g, edge.attr))) {
            if (verbose) message(paste("no edge attribute with name '", edge.attr, "' found so all edge distances set to 1", sep=""))
            g <- set.edge.attribute(g, edge.attr, value=rep(1, ecount(g)))
        }
        
        # check that edge distances are numeric. If not try to convert. If conversion is not possible, return error. 
        if (!is.numeric(get.edge.attribute(g, edge.attr))) {
            edge.distances <- as.numeric(get.edge.attribute(g, edge.attr))
            if (sum(is.na(edge.distances)) > 0) stop("unable to convert non-numeric edge distances to numerals")
            g <- remove.edge.attribute(g, edge.attr)
            g <- set.edge.attribute(g, edge.attr, value=edge.distances)
            warning("non-numeric edge distances found and converted to numerals")
        }
        
        # if negative edge distances present, return error
        if (!all(get.edge.attribute(g, edge.attr) >= 0)) stop("edge distances contain negative values")
        
        # ensure edge distances range between 0 and 1
        edge.distances <- get.edge.attribute(g, edge.attr)
        g <- set.edge.attribute(g, edge.attr, value=edge.distances / max(edge.distances))
    }
    
    g
}

test_CheckAttributes <- function() {
    # test that CheckAttributes produces an error if not all vertex attributes present on graph
    g <- erdos.renyi.game(10, 0.5)
    g <- set.vertex.attribute(g, name="attr1", value=runif(vcount(g)))
    checkException(res <- SANTA:::CheckAttributes(g, vertex.attr=c("attr1", "attr2")), silent=TRUE)
    g <- remove.vertex.attribute(g, "attr1")
    
    # test that CheckAttributes successfully converts non-numeric vertex weights
    values <- runif(vcount(g))
    values[1] <- "1"
    g <- set.vertex.attribute(g, name="vertex.attr", value=values)
    suppressWarnings(suppressMessages(g <- SANTA:::CheckAttributes(g, vertex.attr="vertex.attr")))
    checkEquals(get.vertex.attribute(g, name="vertex.attr")[1], 1)
    g <- remove.vertex.attribute(g, "vertex.attr")
    
    # test that Check Attributes successfully converts logical vertex weights
    values <- c(TRUE, TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, FALSE, TRUE, TRUE)
    g <- set.vertex.attribute(g, name="vertex.attr", value=values)
    suppressWarnings(suppressMessages(g <- SANTA:::CheckAttributes(g, vertex.attr="vertex.attr")))
    checkEquals(get.vertex.attribute(g, name="vertex.attr"), c(1, 1, 1, 1, 0, 0, 0, 0, 1, 1))
    g <- remove.vertex.attribute(g, "vertex.attr")
    
    # test that Check Attributes successfully converts non-numeric "NA"-containing vertex weights
    values <- c("1", "NA", "3", "4", "5", "6", "7", "8", "9", "NA")
    g <- set.vertex.attribute(g, name="vertex.attr", value=values)
    suppressWarnings(suppressMessages(g <- SANTA:::CheckAttributes(g, vertex.attr="vertex.attr")))
    checkEquals(get.vertex.attribute(g, name="vertex.attr"), c(1, NA, 3, 4, 5, 6, 7, 8, 9, NA))
    g <- remove.vertex.attribute(g, "vertex.attr")
    
    # test that CheckAttributes produces an error if it is unable to convert non-numeric vertex weights
    values <- runif(vcount(g))
    values[1] <- "a"
    g <- set.vertex.attribute(g, name="vertex.attr", value=values)
    checkException(SANTA:::CheckAttributes(g, vertex.attr="vertex.attr"), silent=TRUE)
    g <- remove.vertex.attribute(g, "vertex.attr")
    
    # test that CheckAttributes produces an error if not all vertex weights are greater or equal to 0
    values <- runif(vcount(g))
    values[1] <- -1
    g <- set.vertex.attribute(g, name="vertex.attr", value=values)
    checkException(res <- SANTA:::CheckAttributes(g, vertex.attr="vertex.attr"), silent=TRUE)
    g <- remove.vertex.attribute(g, "vertex.attr")
    
    # test that CheckAttributes successfully handles NA-containing vertex weights
    values <- c(1, NA, 3, 4, 5, 6, 7, 8, 9, NA)
    g <- set.vertex.attribute(g, name="vertex.attr", value=values)
    suppressWarnings(suppressMessages(g <- SANTA:::CheckAttributes(g, vertex.attr="vertex.attr")))
    checkEquals(get.vertex.attribute(g, name="vertex.attr"), c(1, NA, 3, 4, 5, 6, 7, 8, 9, NA))
    g <- remove.vertex.attribute(g, "vertex.attr")
    
    # test that CheckAttributes sets missing edge distances to 1
    values <- runif(vcount(g))
    g <- set.vertex.attribute(g, name="vertex.attr", value=values)
    suppressMessages(g <- SANTA:::CheckAttributes(g, vertex.attr="vertex.attr", edge.attr="edge.attr"))
    checkEquals(get.edge.attribute(g, "edge.attr"), rep(1, ecount(g)))
    g <- remove.edge.attribute(g, "edge.attr")
    
    # test that CheckAttributes successfully converts non-numeric edge weights
    values <- runif(ecount(g))
    values[1] <- "1"
    g <- set.edge.attribute(g, name="edge.attr", value=values)
    suppressWarnings(suppressMessages(g <- SANTA:::CheckAttributes(g, vertex.attr="vertex.attr", edge.attr="edge.attr")))
    checkEquals(is.numeric(get.edge.attribute(g, name="edge.attr")[1]), TRUE)
    g <- remove.edge.attribute(g, "edge.attr")
    
    # test that CheckAttributes produces an error if it is unable to convert non-numeric edge weights
    values <- runif(ecount(g))
    values[1] <- "a"
    g <- set.edge.attribute(g, name="edge.attr", value=values)
    suppressWarnings(checkException(res <- SANTA:::CheckAttributes(g, vertex.attr="vertex.attr", edge.attr="edge.attr"), silent=TRUE))
    g <- remove.edge.attribute(g, "edge.attr")
    
    # test that CheckAttributes produces an error if not all edge distances are greater or equal to 0
    values <- runif(ecount(g))
    values[1] <- -1
    g <- set.edge.attribute(g, name="edge.attr", value=values)
    suppressWarnings(checkException(res <- SANTA:::CheckAttributes(g, vertex.attr="vertex.attr", edge.attr="edge.attr"), silent=TRUE))
    g <- remove.edge.attribute(g, "edge.attr")
    
    # test that CheckAttributes ensures all edge.attributes range between 0 and 1 (min != 0 but max == 1)
    values <- runif(ecount(g))
    g <- set.edge.attribute(g, name="edge.attr", value=values)
    g <- SANTA:::CheckAttributes(g, vertex.attr="vertex.attr", edge.attr="edge.attr")
    checkTrue(min(get.edge.attribute(g, "edge.attr")) >= 0)
    checkTrue(max(get.edge.attribute(g, "edge.attr")) == 1)
}

Knode <- function(
    g, 
    dist.method=c("shortest.paths", "diffusion", "mfpt"), 
    vertex.attr="pheno", 
    edge.attr=NULL,
    correct.factor=1,
    nsteps=1000,
    B=NULL, 
    verbose=TRUE
) {  
    # rank the genes exhibiting the phenotype by Knode AUK score. Also produce and list a number of other score for each vertex
    
    # setup
    dist.method <- match.arg(dist.method)
    g <- CheckAttributes(g, vertex.attr, edge.attr, verbose) # check that vertex weights and edge distances are present and suitable
    nvertices <- as.integer(vcount(g))
    if (is.null(get.vertex.attribute(g, "name"))) g <- set.vertex.attribute(g, "name", value=as.character(seq_len(nvertices))) # if vertices do not have a name, add some
    
    # compute D and B if required
    if (is.null(B)) {
        D <- DistGraph(g, edge.attr=edge.attr, dist.method=dist.method, correct.inf=TRUE, correct.factor=correct.factor, verbose=verbose) # compute the vertex pair distances (D)
        B <- BinGraph(D, nsteps=nsteps, verbose=verbose) # compute which bin each vertex pair distance falls into (B)
        rm(D)
    } else {
        if (!identical(dim(B), rep(nvertices, 2))) stop("B is not of the correct dimensions")
    }
    
    # convert to B a vector and compute the maximum of B
    Bv <- as.integer(as.vector(B))
    maxB <- as.integer(max(B))
    rm(B)
    
    # the results for each vertex attrbitute are saved as a data frame. Theses data frames are stored in a list
    res <- vector("list", length(vertex.attr))
    names(res)	<- vertex.attr
    
    # compute the Knode scores for each of the vertex attributes in vertex.attr
    for (attr in vertex.attr) {
        if (verbose) message("running on vertex attribute ", which(vertex.attr == attr), "/", length(vertex.attr), "... ", appendLF=FALSE)
        
        # extract the vertex weights, set any missing weights to the mean
        vertex.weights <- get.vertex.attribute(g, attr)
        vertex.weights[is.na(vertex.weights)] <- mean(vertex.weights, na.rm=TRUE)
        vertex.weights <- as.double(vertex.weights)
        
        # compute nodeAUKs
        nodeAUK <- .Call("computenodeAUK", Bv, vertex.weights, nvertices, maxB)
        names(nodeAUK) <- get.vertex.attribute(g, "name")
        res[[attr]] <- nodeAUK[order(nodeAUK, runif(length(nodeAUK)), decreasing=TRUE)] # sort the Knode scores
        
        if (verbose) message("done")
    }
    
    # if only one vertex attribute is input, don't return a list of lists of results
    if (length(vertex.attr) == 1) res <- res[[1]]
    res
} 

test_Knode <- function() {
    # setup 
    # graph used:
    # 2 - 1 - 4 - 5
    #     | 
    #     3
    
    g <- graph.empty(5, directed=FALSE)
    g <- add.edges(g, c(1, 2, 1, 3, 1, 4, 4, 5))
    g <- set.vertex.attribute(g, "vweights1", value=c(1, 0, 0, 1, 0))
    g <- set.vertex.attribute(g, "vweights2", value=c(0, 1, 0, 0, 1))
    g <- set.edge.attribute(g, "eweights1", value=c(0.01, 1, 1, 1))
    g <- set.edge.attribute(g, "eweights2", value=c(1, 0.01, 1, 1))
    
    # test that Knode correctly uses different distance methods
    dist.methods <- c("shortest.paths", "diffusion", "mfpt")
    for (dist.method in dist.methods) {
        res1 <- Knode(g, dist.method=dist.method, vertex.attr="vweights1", verbose=FALSE)
        checkTrue(names(res1)[1] == "4")
    }
    
    # test that Knode correctly incorperates different vertex weights correctly when using different distance methods
    dist.methods <- c("shortest.paths", "diffusion", "mfpt")
    for (dist.method in dist.methods) {
        res2 <- Knode(g, dist.method=dist.method, vertex.attr="vweights1", verbose=FALSE)
        res3 <- Knode(g, dist.method=dist.method, vertex.attr="vweights2", verbose=FALSE)
        checkTrue(all(as.character(c(1,4)) %in% names(res2)[1:2]))
        checkTrue(all(as.character(c(2,5)) %in% names(res3)[1:2]))
    }
    
    # test that Knode correctly incorperates different edge distances when using the different distance measures
    dist.methods <- c("shortest.paths", "diffusion", "mfpt")
    for (dist.method in dist.methods) {
        res4 <- Knode(g, dist.method=dist.method, vertex.attr="vweights1", edge.attr="eweights1", verbose=FALSE)
        res5 <- Knode(g, dist.method=dist.method, vertex.attr="vweights1", edge.attr="eweights2", verbose=FALSE)
        order4 <- names(res4)
        order5 <- names(res5)
        checkTrue(which(order4=="2") < which(order4=="3") & which(order5=="3") < which(order5=="2"))
    }
        
    # test that Knode outputs a list of lists when multiple vertex attributes are input
    vertex.attributes <- c("vweights1", "vweights2")
    res8 <- Knode(g, vertex.attr=vertex.attributes, verbose=FALSE)
    checkEquals(length(res8), length(vertex.attributes))
}

AsiGraph <- function(
    v, 
    g
) {
    # convert vertex indices to an igraph object
    if (is(v, "igraph.vs")) return(v)
    class(v) <- "igraph.vs"            
    ne <- new.env()
    assign("graph", g, envir = ne)
    attr(v, "env") <- ne
    v
}

test_AsiGraph <- function() {
    # setup
    g <- erdos.renyi.game(5, 1)
    v <- c(1, 2, 3)
    res <- SANTA:::AsiGraph(v, g)
    
    checkEquals(class(res), "igraph.vs") # test that AsiGraph correctly returns vector an igraph.vs object
    checkEquals(length(res), length(v)) # test that AsiGraph correctly returns an igraph.vs object of length x if a vector of lenght x is input
    checkEquals(SANTA:::AsiGraph(res), res) # test that AsiGraph returns object without modificaction if already an igraph.vs object
}

Shuffle <- function(
    x, 
    ignore=NULL
) {
    # shuffle the values in x
    # ignore should be a logical vector equal in length to x
    # for each T value in ignore, the corresponding value in x is not shuffled
    if (is.null(ignore)) ignore <- rep(FALSE, length(x))
    x[!ignore] <- sample(x[!ignore])
    x    
}

test_Shuffle <- function() {
    # setup
    x <- 1:5
    x.na <- c(1, 2, 3, NA, 5)
    n.trials <- 5
    
    for (i in 1:n.trials) {
        # check that function ingores certain values
        ignore1 <- c(FALSE, TRUE, FALSE, TRUE, FALSE)
        res1 <- SANTA:::Shuffle(x, ignore=ignore1)
        checkEquals(x[ignore1], res1[ignore1])
        
        # check that function can ignore all values
        ignore2 <- rep(TRUE, 5)
        res2 <- SANTA:::Shuffle(x, ignore=ignore2)
        checkEquals(x[ignore2], res2[ignore2])
        
        # check that function handles NAs
        ignore3 <- c(FALSE, TRUE, FALSE, TRUE, FALSE)
        res3 <- SANTA:::Shuffle(x.na, ignore=ignore3)
        checkEquals(x.na[ignore3], res3[ignore3])
    }
}

BinGraph <- function(
    D, 
    nsteps=1000,
    equal.bin.fill=TRUE,
    verbose=TRUE
) {
    # split a distance matrix into bins
    
    # setup
    if (verbose) message("computing graph distance bins... ", appendLF=FALSE)
    if (!isSymmetric(D)) stop("D is not symmetric")
    if (nsteps < 2) stop("nsteps is < 2")
    Dv <- D[lower.tri(D, diag=TRUE)] # using only the lower triangle speeds computation
    
    if (length(unique(Dv)) <= nsteps) {
        # if there are fewer than nsteps + 1 unique D values, use each unique value as a break
        breaks <- unique(c(-1, 0, sort(unique(Dv))))
    } else {
        # if there are more unique distances than steps, compute suitable breaks
        
        if (equal.bin.fill) {
            # identify breaks that create bins with rougly equal numbers of nodes
            Dv <- sort(Dv)
            
            # place breaks along the sorted distances
            # if there are duplicate breaks, additional breaks are added, until there is the desired number of unique breaks
            n.breaks <- nsteps + 1 # the number of breaks introduced (may contain duplicates)
            breaks.unique <- numeric(0)
            while (length(breaks.unique) < nsteps + 1) {
                breaks <- unique(c(-Inf, 0, Dv[ceiling(seq(1, length(Dv), length.out=n.breaks)[-1])])) # always include 0 as a break
                breaks[length(breaks)] <- Inf
                breaks.unique <- unique(breaks)
                if (length(breaks.unique) < nsteps + 1) n.breaks <- 2 * n.breaks - length(breaks.unique) 
                if (length(breaks.unique) > nsteps + 1) {
                    values <- breaks.unique[is.finite(breaks.unique) & breaks.unique != 0]
                    samples <- sample(values, nsteps - 2)
                    breaks <- sort(c(-Inf, 0, Inf, samples))
                }
            }
        } else {
            # equally split breaks across the range
            breaks <- c(-Inf, seq(0, max(Dv), length.out=nsteps-1))
        }
    }
    
    # use the breaks to create binned distance matrix
    breaks[length(breaks)] <- Inf
    B <- array(as.numeric(cut(D, breaks=breaks, include.lowest=TRUE, right=TRUE)), dim=dim(D))
    
    # format B
    B <- apply(B, 2, as.integer)
    dimnames(B) <- dimnames(D)
    
    if (verbose) message("done")
    B    
}

test_BinGraph <- function() {
    # TESTS
    # 1) works when D values are all 0
    # 2) works when D values are all 1
    # 3) works when D values are all either 0 or 1
    # 4) works when the number of unique D values is nsteps - 1
    # 5) works when the number of unique D values is == nsteps
    # 6) works when the number of unique D values is nsteps + 1
    # 7) works when all D values are unique
    # 8) works when all but the start D values are unique
    # 9) works when all but the middle D values are unique
    # 10) works when all but the end D values are unique
    # 11) returned B has same dimnames as D
    # 12) produces error when nsteps < 2
    # 13) produces error when D is not symmetric
    # 14) works with large distances within D are present
    
    # 1, 11) 
    n.rowcol <- 4
    names.rowcol <- LETTERS[1:n.rowcol]
    D <- array(0, dim=c(n.rowcol, n.rowcol), dimnames=list(names.rowcol, names.rowcol))
    B <- array(apply(D + 1, 2, as.integer), dim=dim(D), dimnames=dimnames(D))
    checkIdentical(BinGraph(D, nsteps=4), B)
    checkIdentical(dimnames(BinGraph(D, nsteps=4)), dimnames(D))
    
    # 2) 
    D <- matrix(1, 4,4)
    diag(D) <- 0
    B <- array(apply(D + 1, 2, as.integer), dim=dim(D), dimnames=dimnames(D))
    checkIdentical(BinGraph(D, nsteps=4), B)
    
    # 3)
    D <- matrix(1, 4,4)
    diag(D) <- 0
    D[1,4] <- D[4,1] <- 0
    B <- array(apply(D + 1, 2, as.integer), dim=dim(D), dimnames=dimnames(D))
    checkIdentical(BinGraph(D, nsteps=4), B)
    
    # 4,5,6)
    D <- matrix(c(0,1,2,3,1,0,1,2,2,1,0,1,3,2,1,0), 4,4)
    B <- array(apply(D + 1, 2, as.integer), dim=dim(D), dimnames=dimnames(D))
    checkIdentical(BinGraph(D, nsteps=5), B) # 4)
    checkIdentical(BinGraph(D, nsteps=4), B) # 5)
    B <- matrix(as.integer(c(1,2,3,3,2,1,2,3,3,2,1,2,3,3,2,1)), 4,4)
    checkIdentical(BinGraph(D, nsteps=3), B) # 6)
    
    # 7) 
    D <- matrix(c(0,1,2,3,4,1,0,5,6,7,2,5,0,8,9,3,6,8,0,10,4,7,9,10,0), 5,5)
    B <- matrix(as.integer(c(1,2,2,2,3,2,1,3,3,3,2,3,1,4,4,2,3,4,1,4,3,3,4,4,1)), 5,5)
    checkIdentical(BinGraph(D, nsteps=4), B)
    
    # 8,9,10)
    nrow.tot <- 8 # number of rows/columns
    nrow.na <- 6 # size of NA square to be replaced with value
    D.template <- array(runif(nrow.tot ^ 2), dim=c(nrow.tot, nrow.tot))
    D.template <- (D.template + t(D.template)) / 2 # ensure D is symmetric
    D.template[cbind(rep(1:nrow.na, nrow.na), rep(1:nrow.na, each=nrow.na))] <- NA
    diag(D.template) <- 0
    nsteps <- 5
    values.na <- c(10e-10, 0.5, 1)
    for (value in values.na) {
        # check that each of the steps is represented, no matter where the group of tied vertices comes
        D <- D.template
        D[is.na(D)] <- value
        B <- BinGraph(D, nsteps)
        checkIdentical(1:nsteps, sort(unique(as.vector(B)))) 
    }
    
    # 12)
    D <- matrix(1, 4,4)
    diag(D) <- 0
    checkException(BinGraph(D, 1), silent=TRUE)
    
    # 13)
    D <- matrix(1, 4,4)
    D[1,2] <- 2
    diag(D) <- 0
    checkException(BinGraph(D, 1), silent=TRUE)
    
    # 14) 
    nsteps <- 5
    D <- matrix(10^(1:16), 4,4)
    D <- D + t(D)
    diag(D) <- 0
    B <- BinGraph(D, nsteps)
    checkEquals(min(B), 1)
    checkEquals(max(B), nsteps)
}

Compactness <- function(
    g, 
    nperm=100, 
    dist.method=c("shortest.paths", "diffusion", "mfpt"), 
    vertex.attr="pheno",
    edge.attr="distance", # an attribute containing the edge distance
    correct.factor=1,
    D=NULL,
    verbose=TRUE
) {
    # use the compactness method to measure the strength of association between a gene set and a network
    # this method is based upon the PathExpand method by Glaab et al. 
    
    # check that vertex weights and edge distances are present and suitable. Convert if neccessary
    dist.method <- match.arg(dist.method)
    g <- CheckAttributes(g, vertex.attr, edge.attr, verbose) 
   
    # produce list to hold results
    element.names <- c("score.obs", "score.perm", "pval") 
    res.attr <- rep(list(NA), length(element.names))
    names(res.attr) <- element.names    
    res <- rep(list(res.attr), length(vertex.attr))
    names(res) <- vertex.attr
    
    # compute D if not input
    if (is.null(D)) {
        D <- DistGraph(g, edge.attr=edge.attr, dist.method=dist.method, correct.inf=TRUE, correct.factor=correct.factor, verbose=verbose) 
    } else {
        if (nrow(D) != vcount(g) | ncol(D) != vcount(g)) stop("D does not have the correct dimensions")
    }
    
    # compute the compactness scores for each vertex attribute
    for (attr in vertex.attr) {
        # compute the observed compactness score
        
        # check the weights
        vertex.weights <- get.vertex.attribute(g, attr) 
        if (!all(vertex.weights %in% c(0, 1))) stop("vertex weights not binary")
        hits <- which(vertex.weights == 1)
        res[[attr]][["score.obs"]] <-  mean(D[hits, hits][lower.tri(D[hits, hits], diag=FALSE)])
        
        if (nperm > 0) {
            # permute the hits and compute the permuted compactness scores
            res[[attr]][["score.perm"]] <- rep(0, nperm)
            for (i in seq_len(nperm)) {
                hits.perm <- sample(vcount(g), length(hits))
                res[[attr]][["score.perm"]][i] <- mean(D[hits.perm, hits.perm][lower.tri(D[hits.perm, hits.perm], diag=FALSE)])
            }
            
            # use the z-test to produce a p-value
            res[[attr]][["pval"]] <-  pnorm((res[[attr]][["score.obs"]] - mean(res[[attr]][["score.perm"]])) / sd(res[[attr]][["score.perm"]]), lower.tail=TRUE) 
        }
    }
    
    # output
    if (length(vertex.attr) == 1) res <- res[[1]] # if only a single vertex attr input, don't return list
    res   
}

test_Compactness <- function() {
    # setup 
    # graph used:
    # 2 - 1 - 4 - 5
    #     | 
    #     3
    
    g <- graph.empty(5, directed=FALSE)
    g <- add.edges(g, c(1,2,1,3,1,4,4,5))
    g <- set.vertex.attribute(g, "vw1", value=c(1, 0, 0, 1, 0))
    g <- set.vertex.attribute(g, "vw2", value=c(0, 1, 0, 0, 1))
    g <- set.edge.attribute(g, "ew1", value=c(1, 1, 0.01, 1))
    g <- set.edge.attribute(g, "ew2", value=c(0.01, 0.01, 1, 0.01))
    
    # correctly incorperates different binary vertex weights 
    res1 <- Compactness(g, nperm=1000, vertex.attr="vw1")
    res2 <- Compactness(g, nperm=1000, vertex.attr="vw2")
    checkEquals(res1$score.obs, 1)
    checkEquals(res2$score.obs, 3)
    checkTrue(res1$pval < res2$pval)
    
    # correctly incorperates different edge distances when using the different distance measures
    dist.methods <- c("shortest.paths", "diffusion", "mfpt")
    for (dist.method in dist.methods) {
        res3 <- Compactness(g, nperm=1000, dist.method=dist.method, vertex.attr="vw1", edge.attr="ew1")
        res4 <- Compactness(g, nperm=1000, dist.method=dist.method, vertex.attr="vw1", edge.attr="ew2")
        checkTrue(res3$score.obs < res4$score.obs)
        checkTrue(res3$pval < res4$pval)
    }
    
    # outputs a list of results containing the correct elements when one vertex attribute is input
    res5 <- Compactness(g, nperm=10, vertex.attr="vw1", edge.attr="ew1")
    checkEquals(names(res5), c("score.obs", "score.perm", "pval") )
    
    # outputs a list of lists of results containing the correct elements when multiple vertex attributes are input
    vertex.attributes <- c("vw1", "vw2")
    res6 <- Compactness(g, nperm=10, vertex.attr=vertex.attributes, edge.attr="ew1")
    checkEquals(length(res6), length(vertex.attributes))
    for (r in res6) checkEquals(names(r), c("score.obs", "score.perm", "pval"))
    
    # outputs a list of results, some of which equal NA, when one attribute and no permutations are completed
    res7 <- Compactness(g, nperm=0, vertex.attr="vw1", edge.attr="ew1")
    checkTrue(is.na(res7$score.perm))
    checkTrue(is.na(res7$pval))
}

plot.Knet <- function(
  x, 
  sequential=FALSE,
  ...
){
  # plot the Knet score

  if (all(is.na(x$K.perm), is.na(x$AUK.perm), is.na(x$K.quan))){
    # if no permutations present, plot just the K function line
    plot(x$K.obs, main="Knet function", type="l", lwd=3, col="red", xlab="distance on graph", ylab="Knet")
  } else {
    # if permutations present, produce 2 plots comparing the observed and permutation K function 
    if (!sequential) par(mfrow=c(1, 2))

    # extract Scores
    K.obs    <- x$K.obs
    K.perm   <- x$K.perm
    K.quan   <- x$K.quan
    AUK.obs  <- x$AUK.obs
    AUK.perm <- x$AUK.perm
    g.max	 <- ncol(K.quan)-1
    
    # plot 1
    if (nrow(K.quan) == 5) { 
      lty <- c("dotted", "solid", "solid", "solid", "dotted") 
      lwd <- c(1, 1, 2, 1, 1)
    } else {
      lty	<- rep("solid", nrow(K.quan))
      lwd <- rep(1, nrow(K.quan))
    }
    plot(NA, ylim=range(K.obs, K.quan), xlim=c(0, g.max), xlab="Distance in graph", ylab="Knet", main="Knet-function", ...)
    polygon(x=c(0:g.max, g.max:0), y=c(K.quan["0%",], rev(K.quan["100%",])), col="lightyellow", border=NA)
    for (i in seq_len(nrow(K.quan))) lines(0:(ncol(K.quan)-1), K.quan[i,], col="grey", lty=lty[i], lwd=lwd[i])
    lines(x=0:g.max, y=K.obs, col="red", lwd="3")
	
    # plot 2
    xlim.values <- range(c(AUK.perm, AUK.obs)) + 1 * (range(c(AUK.perm, AUK.obs)) - mean(range(c(AUK.perm, AUK.obs)))) 
    hist(AUK.perm, xlim=xlim.values, col="grey", border="grey", xlab="Area under K-curve", main="AUK: observed v. permuted")
    abline(v=AUK.obs, lwd=2, col="red")
	}
}

CreateGrid <- function(
    n=100
) {
    # create a grid-like graph containing n vertices
    nr <- floor(sqrt(n))
    g <- graph.empty(n, directed=FALSE) 
    g <- add.edges(g, rbind(seq_len(n-1), 2:(n))[, c(rep(TRUE, nr-1), FALSE)]) # add edges from i to i+1 (within layer)
    g <- add.edges(g, as.numeric(rbind(seq_len(n-nr), (nr+1):n))) # add edges between layers
    g
}

test_CreateGrid <- function() {
    # test that function creates a square graph with the correct number of edges
    g <- CreateGrid(9)
    checkEquals(vcount(g), 9)
    checkEquals(ecount(g), 12)
    
    # test that function creates a non-square graph with the correct number of edges
    g <- CreateGrid(11)
    checkEquals(vcount(g), 11)
    checkEquals(ecount(g), 15)
}

## options(width = 75)
## options(useFancyQuotes=FALSE)
## 

## # generate the simulated network
## require(SANTA, quietly=TRUE)
## require(igraph, quietly=TRUE)
## set.seed(1) # for reproducibility
## g <- barabasi.game(n=500, power=1, m=1, directed=FALSE)

## # measure the distance between pairs of vertices in g
## dist.method <- "shortest.paths"
## D <- DistGraph(g, dist.method=dist.method, verbose=FALSE)

## # place the distances into discreet bins
## B <- BinGraph(D, verbose=FALSE)

## cluster.size <- 5
## s.to.use <- c(10, 20, 50, 100, 500)
## n.trials <- 10
## pvalues <- array(0, dim=c(n.trials, length(s.to.use)),
##     dimnames=list(NULL, as.character(s.to.use)))
## 
## # run the trials for each value of s
## for (s in s.to.use) {    	
##     for (i in 1:n.trials) {
##         # generate the hit set
##         seed.vertex <- sample(vcount(g), 1) # identify seed
##         sample.set <- order(D[seed.vertex, ])[1:s]
##         hit.set <- sample(sample.set, cluster.size)
## 
##         # measure the stength of association
##         g <- set.vertex.attribute(g, name="hits",
##             value=as.numeric(1:vcount(g) %in% hit.set))
##         pvalues[i, as.character(s)] <- Knet(g, nperm=100,
##             dist.method=dist.method, vertex.attr="hits",
##             B=B, verbose=FALSE)$pval
##     }
## }

## boxplot(-log10(pvalues), xlab="cutoff", ylab="-log10(p-value)")

## # cleanup
## rm(B, D, g)

## # create the network
## n.nodes <- 12
## edges <- c(1,2, 1,3, 1,4, 2,3, 2,4, 3,4, 1,5, 5,6,
##            2,7, 7,8, 4,9, 9,10, 3,11, 11,12)
## weights1 <- weights2 <- rep(0, n.nodes)
## weights1[c(1,2)] <- 1
## weights2[c(5,6)] <- 1
## 
## g <- graph.empty(n.nodes, directed=FALSE)
## g <- add.edges(g, edges)
## g <- set.vertex.attribute(g, "weights1", value=weights1)
## g <- set.vertex.attribute(g, "weights2", value=weights2)

## par(mfrow=c(1,2))
## 
## colors <- rep("grey", n.nodes)
## colors[which(weights1 == 1)] <- "red"
## g <- set.vertex.attribute(g, "color", value=colors)
## plot(g)
## 
## colors <- rep("grey", n.nodes)
## colors[which(weights2 == 1)] <- "red"
## g <- set.vertex.attribute(g, "color", value=colors)
## plot(g)
## 
## par(mfrow=c(1,1))
## g <- remove.vertex.attribute(g, "color")

## # set 1
## Knet(g, nperm=100, vertex.attr="weights1", verbose=FALSE)$pval
## Compactness(g, nperm=100, vertex.attr="weights1", verbose=FALSE)$pval
## 
## # set 2
## Knet(g, nperm=100, vertex.attr="weights2", verbose=FALSE)$pval
## Compactness(g, nperm=100, vertex.attr="weights2", verbose=FALSE)$pval

## # load igraph objects
## data(g.costanzo.raw)
## data(g.costanzo.cor)
## networks <- list(raw=g.costanzo.raw, cor=g.costanzo.cor)
## network.names <- names(networks)
## network.genes <- V(networks$raw)$name
## # genes identical across networks

## # obtain the GO term associations from org.Sc.sgd.db package
## library(org.Sc.sgd.db, quietly=TRUE)
## xx <- as.list(org.Sc.sgdGO2ALLORFS)
## go.terms <- c("GO:0000082", "GO:0003682", "GO:0007265",
##               "GO:0040008", "GO:0090329")
## # apply the GO terms to the networks
## for (name in network.names) {
##     for (go.term in go.terms) {
##         networks[[name]] <- set.vertex.attribute(
##             networks[[name]], name=go.term,
##             value=as.numeric(network.genes %in% xx[[go.term]]))
##     }
## }

## results <- list()
## for (name in network.names) {
##     results[[name]] <- Knet(networks[[name]], nperm=10,
##         vertex.attr=go.terms, edge.attr="distance", verbose=FALSE)
##     results[[name]] <- sapply(results[[name]],
##         function(res) res$pval)
## }

## # cleanup
## rm(g.costanzo.cor, g.costanzo.raw, network.genes, networks, xx)

## # load igraph objects
## data(g.bandyopadhyay.treated)
## data(g.bandyopadhyay.untreated)
## networks <- list(
##     treated=g.bandyopadhyay.treated,
##     untreated=g.bandyopadhyay.untreated
## )
## network.names <- names(networks)

## # obtain GO term associations
## library(org.Sc.sgd.db, quietly=TRUE)
## xx <- as.list(org.Sc.sgdGO2ALLORFS)
## # change to use alternative GO terms
## associated.genes <- xx[["GO:0006974"]]
## associations <- sapply(networks, function(g)
##     as.numeric(V(g)$name %in% associated.genes),
##     simplify=FALSE)
## networks <- sapply(network.names, function(name)
##     set.vertex.attribute(networks[[name]], "rdds",
##     value=associations[[name]]), simplify=FALSE)

## results <- sapply(networks, function(g) Knet(g, nperm=10,
##     dist.method="shortest.paths", vertex.attr="rdds",
##     edge.attr="distance"), simplify=FALSE)
## p.values <- sapply(results, function(res) res$pval)
## p.values

## # cleanup
## rm(xx, networks, g.bandyopadhyay.treated, g.bandyopadhyay.untreated, associated.genes)

## # laod igraph object
## data(g.srivas.high)
## data(g.srivas.untreated)
## networks <- list(
##     high=g.srivas.high,
##     untreated=g.srivas.untreated
## )
## network.names <- names(networks)

## # obtain GO term associations
## library(org.Sc.sgd.db, quietly=TRUE)
## xx <- as.list(org.Sc.sgdGO2ALLORFS)
## associated.genes <- xx[["GO:0000725"]]
## associations <- sapply(networks, function(g)
##     as.numeric(V(g)$name %in% associated.genes),
##     simplify=FALSE)
## networks <- sapply(network.names, function(name)
##     set.vertex.attribute(networks[[name]], "dsbr",
##     value=associations[[name]]), simplify=FALSE)

## p.values <- sapply(networks, function(g)
##        Knet(g, nperm=10, dist.method="shortest.paths",
##        vertex.attr="dsbr", edge.attr="distance")$pval)
## p.values

## # cleanup
## rm(xx, networks, g.srivas.high, g.srivas.untreated)

## # import and convert RNAi data
## data(rnai.cheung)
## rnai.cheung <- -log10(rnai.cheung)
## rnai.cheung[!is.finite(rnai.cheung)] <- max(rnai.cheung[is.finite(rnai.cheung)])
## rnai.cheung[rnai.cheung < 0] <- 0

## # import and create IntAct network
## data(edgelist.intact)
## g.intact <- graph.edgelist(as.matrix(edgelist.intact),
##     directed=FALSE)
## 
## # import data and create HumanNet network
## data(edgelist.humannet)
## g.humannet <- graph.edgelist(as.matrix(edgelist.humannet)[,1:2],
##     directed=FALSE)
## g.humannet <- set.edge.attribute(g.humannet, "distance",
##     value=edgelist.humannet$distance)
## networks <- list(intact=g.intact, humannet=g.humannet)

## network.names <- names(networks)
## network.genes <- sapply(networks, get.vertex.attribute,
##     name="name", simplify=FALSE)
## rnai.cheung.genes <- rownames(rnai.cheung)
## cancers <- colnames(rnai.cheung)
## 
## for (cancer in cancers) {
##     for (name in network.names) {
##         vertex.weights <-rep(NA, vcount(networks[[name]]))
##         names(vertex.weights) <- network.genes[[name]]
##         common.genes <- rnai.cheung.genes[rnai.cheung.genes
##             %in% network.genes[[name]]]
##         vertex.weights[common.genes] <- rnai.cheung[common.genes, cancer]
##         networks[[name]] <- set.vertex.attribute(networks[[name]],
##             cancer, value=vertex.weights)
##     }
## }

## knet.res <- sapply(networks, Knet, nperm=10,
##    dist.method="shortest.paths", vertex.attr=cancers,
##    edge.attr="distance", simplify=FALSE)
## p.values <- sapply(knet.res, function(i) sapply(i,
##    function(j) j$pval))
## p.values

## # cleanup
## rm(rnai.cheung, rnai.cheung.genes, networks, network.genes, edgelist.humannet, edgelist.intact, common.genes, g.humannet, g.intact)

## sessionInfo()

SpreadHits <- function(
    g, 
    h, 
    clusters=1, 
    distance.cutoff=3,
    lambda=1, 
    dist.method=c("shortest.paths", "diffusion", "mfpt"), 
    edge.attr=NULL, 
    hit.color="red", 
    D=NULL, 
    attempts=1000,
    verbose=TRUE
) {
    # spread hits across a graph in multiple clusters
    # this is done by trial by error
    
    # setup
    dist.method <- match.arg(dist.method)
    if (!is(g, "igraph")) stop("g is not a graph")
    if (h * clusters > vcount(g)) stop("too few vertices on network to apply this many hits")
    if (lambda < 0) stop("lambda needs to be greater than or equal to 0")
    
    # compute D if not input
    if (is.null(D)) D <- DistGraph(g, edge.attr=edge.attr, dist.method=dist.method, verbose=verbose)
    D.cutoff <- D >= distance.cutoff
      
    # identify the seed vertices
    seed.flag <- FALSE
    attempt.number <- 0
    while (!seed.flag & attempt.number < attempts) {
        # for a certain maximum number of attempt, try to find a suitable number of seed vertices distant appart
        attempt.number <- attempt.number + 1
        potential.seed <- rep(TRUE, vcount(g))
        seeds <- rep(NA, clusters)
        seeds[1] <- sample(vcount(g), 1)
        
        # try to identify seeds that are suitable distant
        while (!all(potential.seed == FALSE) & !all(!is.na(seeds))) {
            seeds.chosen <- sum(!is.na(seeds))
            potential.seed <- if (seeds.chosen == 1) D.cutoff[seeds[seeds.chosen], ] else apply(D.cutoff[seeds[seq_len(seeds.chosen)], ], 2, all)
            if (!all(!potential.seed)) seeds[seeds.chosen + 1] <- sample(which(potential.seed), 1)
        }
        
        # if enough hits have been chosen, proceed
        if (all(!is.na(seeds))) seed.flag <- TRUE
    }
    if (!seed.flag) warning("seed vertices not identified")
    
    if (seed.flag) {
        # spread hits over the network, starting from each seed
        hits <- list()
        for (cluster in seq_len(clusters)) {
            # spread the hits across the graph, probability determined by distance from the start vertex
            prob <- if (lambda > 0) lambda ^ - D[seeds[cluster], ] else rep(1, ncol(D))
            if (sum(prob > 1) > 0) prob <- prob / max(prob) # ensure that no probability are greater than 1
            if (sum(prob > 0) < h + 1) prob[prob == 0] <- min(prob[prob != 0]) # ensure that there aren't too few positive probablility
            prob[seeds] <- 0 # ensure that none of the seeds are chosen
            prob[unlist(hits)] <- 0 # ensure that none of the previously chosen hits are chosen
            
            # choose start.vertex and h-1 other vertices
            hits[[cluster]] <- c(seeds[cluster], sample(as.numeric(V(g)), size=h - 1, prob=prob))
        }
        
        # add the hits and colour to the network
        hit.weights <- rep(0, vcount(g))
        color <- rep("grey", vcount(g))
        hit.weights[seq_len(vcount(g)) %in% unlist(hits)] <- 1
        color[seq_len(vcount(g)) %in% unlist(hits)] <- hit.color
        g <- set.vertex.attribute(g, name="hits", value=hit.weights)
        g <- set.vertex.attribute(g, name="color", value=color)
    } 
     
    # if both the hits have been successfully applied, return the graph
    if (seed.flag) return(g) else return(NULL)
}

test_SpreadHits <- function() {
    # setup 
    # create string of nodes
    n.nodes <- 20
    edges <- rep(1:n.nodes, each=2)[-c(1, n.nodes*2)]
    g <- graph.empty(n.nodes, directed=FALSE)
    g <- add.edges(g, edges)
    g <- set.edge.attribute(g, "distances1", value=rep(10, ecount(g)))
   
   
    # test that SpreadHits adds the correct number of hits across 1, 2 and 3 clusters
    # test that SpreadHits adds vertex attributes "hits" and "color"
    clusters <- 1:3
    h <- 2
    for (cluster in clusters) {
        g1 <- SpreadHits(g, h=h, clusters=cluster, distance.cutoff=2, lambda=10, dist.method="shortest.paths")
        hits <- which(get.vertex.attribute(g1, "hits") == 1)
        
        checkEquals(length(hits), h*cluster)
        checkTrue(identical(sort(list.vertex.attributes(g1)), c("color", "hits")))
    }

    # test that SpreadHits uses specified D when input
    h <- 2
    clusters <- 3
    D <- DistGraph(g, edge.attr="distances1", dist.method="shortest.paths")
    g2 <- SpreadHits(g, h=h, clusters=clusters, distance.cutoff=20, lambda=10, dist.method="shortest.paths", D=D)
    hits <- which(get.vertex.attribute(g2, "hits") == 1)
    checkEquals(length(hits), h*clusters)
    
    # test that SpreadHits produces an error when g is not an igraph object
    checkException(SpreadHits(6), silent=TRUE)
    
    # test that SpreadHits produces an error when lambda is negative
    checkException(SpreadHits(g, h=h, clusters=clusters, distance.cutoff=20, lambda=-10, dist.method="shortest.paths"), silent=TRUE)
    
    # test that SpreadHits produces error if more hits than vertices are specified to be applied
    clusters <- 2
    checkException(SpreadHits(g, h=vcount(g)/clusters + 1, clusters=clusters, distance.cutoff=2, lambda=10, dist.method="shortest.paths", edge.attr=NULL), silent=TRUE)
    
    # test that SpreadHits returns NULL if it is not possible to identify seed nodes the required distance apart
    g4 <- SpreadHits(g, h=2, clusters=3, distance.cutoff=20, lambda=10, dist.method="shortest.paths", edge.attr=NULL)
    checkTrue(is.null(g4))
}

GraphDiffusion <- function(
    g,
    v=V(g), 
    edge.attr.weight=NULL,
    beta=1, 
    correct.neg=TRUE
) { 
    # calculates the diffusion distance using the diffusion kernel method
    # Diffusion Kernels on Graphs and Other Discrete Structures, Kondor and Lafferty, 2002
    # the named numeric distance matrix is returned
    # any edge attribute specified should be the weight of the edge (higher weights -> more significant) not the distance
    
    if (!is(v, "igraph.vs")) v <- AsiGraph(v, g) # if v is not an igraph-class object, convert
    if (vcount(g) == 1) return(matrix(0, 1, 1, dimnames=list(v$name, V(g)$name))) # if only a single vertex is contained within the graph, return a single cell matrix
    
    # obtain the sparse unamed adjacency matrix
    adj <- get.adjacency(g, attr=edge.attr.weight, names=FALSE, sparse=TRUE, type="both")
    
    # compute the distance matrix for each of the connected clusters of vertices
    D <- array(Inf, dim=rep(vcount(g), 2))
    c <- clusters(g) 
    for (i in seq_len(c$no)) {
        indices <- which(c$membership == i)
        D[indices, indices] <- Diffusionfct(adj[indices, indices], beta, correct.neg)
    }
    diag(D) <- 0 # ensure that the diagonal is 0 
    
    # return rows for each vertex in v
    D <- as.matrix(D[v, ])
    dimnames(D) <- list(V(g)$name[as.numeric(v)], V(g)$name) 
    
    D
}



Diffusionfct <- function(
    adj,
    beta,
    correct.neg
) {
    # calculates the diffusion-kernel based distance matrix for an adjacency matrix
    
    # if the adjacency matrix contains only a single gene, return a 1x1 matrix containing 0
    if (is.null(dim(adj))) return(matrix(0, 1, 1))
    
    # compute the diffusion kernal
    H <- adj - Diagonal(x=apply(adj, 1, sum))
    x <- eigen(H, symmetric=TRUE)
    K  <- x$vectors %*% diag(exp(beta * x$values)) %*% t(x$vectors)
    Dsub <- outer(diag(K), diag(K), "+") - 2*K
    
    # correct negative distances
    if (any(Dsub < 0) && correct.neg){
        warning("negative distances set to zero")
        Dsub[Dsub < 0] <- 0
    }
    
    sqrt(Dsub)
}

test_GraphDiffusion <- function() { 
    # TESTS 
    # 1) returns matrix of dim 1,1 if networks with 1 vertex input
    # 2) uses v vertex names (under the attribute name 'name') as row names if available
    # 3) uses g vertex names (under the attribute name 'name') as column names if available
    # 4) returns only the rows corresultsponding to v, in the order v was input
    # 5) correctly incorperates edge weights
    # 6) doesn't return negative distances when correct.neg is TRUE
    # 7) returns the correct results on a small test graph
    
    # setup
    g.single <- graph.empty(1, directed=FALSE)
    
    edge.attr.weight <- "weight"
    edges <- c(1,4, 1,8, 1,9, 1,10, 1,11, 2,5, 2,6, 2,9, 3,7, 3,10, 3,12, 4,2, 4,8, 4,11, 5,3, 5,6, 5,9, 6,9, 7,10, 7,11, 7,12, 8,11, 9,10, 10,12) 
    weights <- rep(1, length(edges) / 2)
    weights[c(2, 7, 13, 16, 18, 22)] <- 0.1 # move 8 and 6 away
    g <- graph.empty(max(edges), directed=FALSE)
    g <- add.edges(g, edges)
    g <- set.edge.attribute(g, edge.attr.weight, value=weights)
    g.unnamed <- g
    g.named <- set.vertex.attribute(g, "name", value=paste("gene", 1:vcount(g), sep=""))
    
    n.vertex.unconnected <- 10
    g.unconnected <- graph.empty(n.vertex.unconnected, directed=FALSE)
    
    v <- c(4,2,12)
    
    
    # run function
    results <- list()
    results[[1]] <- GraphDiffusion(g.single) # 1 vertex
    results[[2]] <- GraphDiffusion(g.unnamed) # >1 vertex, v = V(g), no vertex names
    results[[3]] <- GraphDiffusion(g.named) # >1 vertex, v = V(g), vertex names
    results[[4]] <- GraphDiffusion(g.named, v=v) # >1 vertex, v = subset, vertex names
    results[[5]] <- GraphDiffusion(g.named, edge.attr.weight=edge.attr.weight) # >1 vertex, v = V(g), no vertex name, edge weights
    results[[6]] <- GraphDiffusion(g.unconnected) # unconnected network
    
    
    # conduct tests
    checkTrue(all(dim(results[[1]]) - c(1,1) < 10e-10)) # 1
    
    checkTrue(is.null(rownames(results[[2]]))) # 2
    checkIdentical(rownames(results[[3]]), V(g.named)$name) # 2
    checkIdentical(rownames(results[[4]]), V(g.named)$name[v]) # 2
    
    checkTrue(is.null(colnames(results[[2]]))) # 3
    checkIdentical(colnames(results[[3]]), V(g.named)$name) # 3
    checkIdentical(colnames(results[[4]]), V(g.named)$name) # 3
    
    checkIdentical(results[[4]], results[[3]][v, ]) # 4
    
    checkTrue(all(c(8, 6) %in% tail(order(results[[5]][1, ]), 2))) # 5
    
    for (result in results) checkEquals(sum(result < 0), 0) # 6
    
    for (result in results[c(2, 3)]) {
        checkTrue(all(c(1, 4, 8, 11) %in% head(order(result[1, ]), 4))) # 7
        checkTrue(all(c(2, 5, 6, 9) %in% head(order(result[2, ]), 4))) # 7
        checkTrue(all(c(3, 7, 10, 12) %in% head(order(result[3, ]), 4))) # 7
    }
}

DistGraph <- function(
    g, 
    v=V(g), 
    edge.attr=NULL,
    dist.method=c("shortest.paths", "diffusion", "mfpt"), 
    correct.inf=TRUE,
    correct.factor=1,
    verbose=TRUE
) {
    # compute the distance matrix for a graph using a specified method
    # the attribute specified by edge.attr should represent edge distances
    
    method <- match.arg(dist.method)
    if (verbose) message("computing graph distance matrix... ", appendLF=FALSE)
    if (!is(v, "igraph.vs")) v <- AsiGraph(v, g) # if v is not an igraph object, convect
    
    # convert the distances to weights
    if (is.null(edge.attr)) {
        distances <- rep(1, ecount(g)) # if NULL, then sometimes and error is produced by the shortest.paths function
        edge.attr.weight <- NULL
    } else {
        distances <- get.edge.attribute(g, edge.attr)
        weights <- max(distances) - distances + min(distances)
        g <- set.edge.attribute(g, name="weights_converted_from_distances", value=weights)
        edge.attr.weight <- "weights_converted_from_distances"
    }
    
    # the distance matrix D contains a row for each vertex in v and a column for each vertex in g
    D <- switch(method,
                shortest.paths = shortest.paths(g, v=v, weights=distances),			
                diffusion      = GraphDiffusion(g, v=v, edge.attr.weight=edge.attr.weight, correct.neg=TRUE),
                mfpt           = GraphMFPT(g, v=v, edge.attr.weight=edge.attr.weight, average.distances=TRUE)
    )
    if (sum(is.na(D)) > 0) warning("NA values returned in D")
    
    # if there are unconnected vertex pairs, the function changes the distance to twice the maximum finite distance.
    if (correct.inf) D[!is.finite(D)] <- correct.factor * max(D[is.finite(D)], na.rm=TRUE)
    
    # change column names and row names to the names of the specified vertices
    if (is.null(rownames(D))) rownames(D) <- v$name
    if (is.null(colnames(D))) colnames(D) <- V(g)$name
    if (verbose) message("done")
    D
}

test_DistGraph <- function() {
  # setup
  g <- graph.empty(6, directed=FALSE)
  g <- add.edges(g, c(1, 2, 1, 3, 2, 4, 3, 4, 1, 5, 5, 6))
  g <- set.edge.attribute(g, "weights", value=c(0.1, 0.9, 0.1, 0.9, 0.5, 0.5))
  
  # test that DistGraph correctly returns the correct relative distances using the shortest.paths algorithm
  # without edge weights
  D1 <- DistGraph(g, dist.method="shortest.paths")
  checkEquals(D1[1, 1], 0)
  checkEquals(D1[1, 4], D1[1, 6])
  
  # with edge weights
  D2 <- DistGraph(g, dist.method="shortest.paths", edge.attr="weights")
  checkTrue(all(D2[1, 2] < D2[1, 3]))
  checkTrue(all(D2[4, 2] < D2[4, 3]))
  
  # test that DistGraph correctly returns the correct relative distances using the diffusion algorithm with edge weights
  # without edge weights
  D3 <- DistGraph(g, dist.method="diffusion")
  checkEquals(D3[1, 1], 0)
  checkTrue(D3[1, 4] < D3[1, 6])
  
  # with edge weights
  D4 <- DistGraph(g, dist.method="diffusion", edge.attr="weights")
  checkTrue(all(D4[1, 2] < D4[1, 3]))
  checkTrue(all(D4[4, 2] < D4[4, 3]))
  
  # test that DistGraph correctly returns the correct relative distances using the MFPT algorithm with edge weights
  # without edge weights
  D5 <- DistGraph(g, dist.method="mfpt")
  checkEquals(D5[1, 1], 0)
  checkTrue(D5[1, 4] < D5[1, 6])
  
  # with edge weights
  D6 <- DistGraph(g, dist.method="mfpt", edge.attr="weights")
  checkTrue(all(D6[1, 2] < D6[1, 3]))
  checkTrue(all(D6[4, 2] < D6[4, 3]))
  
  # setup
  g <- graph.empty(7, directed=FALSE)
  g <- add.edges(g, c(1, 2, 1, 3, 2, 4, 3, 4, 1, 5, 5, 6))
  
  # test that DistGraph does and does not correct infinite values when specified
  D7 <- DistGraph(g, dist.method="shortest.paths", correct.inf=FALSE)
  checkTrue(!is.finite(D7[1, 7]))
  D7 <- DistGraph(g, dist.method="shortest.paths", correct.inf=TRUE)
  checkTrue(is.finite(D7[1, 7]))
  
  # test that DistGraph correctly changes infinite values to correct.factor * maximum finite distance measured
  correct.factor <- 1.5
  D8 <- DistGraph(g, dist.method="shortest.paths", correct.inf=TRUE, correct.factor=correct.factor)
  checkEquals(D8[1, 7],correct.factor * (max(D8[D8 != D8[1, 7]])))
  correct.factor <- 4.5
  D9 <- DistGraph(g, dist.method="shortest.paths", correct.inf=TRUE, correct.factor=correct.factor)
  checkEquals(D9[1, 7], correct.factor * (max(D9[D9 != D9[1, 7]])))
  
  # test that DistGraph correctly handles the lack of infinite values on fully connected graphs
  g <- graph.empty(6, directed=FALSE)
  g <- add.edges(g, c(1, 2, 1, 3, 2, 4, 3, 4, 1, 5, 5, 6))
  D10 <- DistGraph(g, dist.method="shortest.paths", correct.inf=TRUE)
  checkTrue(all(is.finite(D10)))
}

ComputeWeightsInBins <- function(
    B, 
    weight, 
    nvertices
){
	# Use C call to compute the sum of the adjusted vertex weights for each vertex and each bin
	diam <- max(B)
	matrix(
		.Call("computeweights",
			as.integer(as.vector(B)),
			as.double(weight),
			as.integer(nvertices),
			as.integer(diam)
		),
	nvertices, diam)
}

test_ComputeWeightsInBins <- function() {
  # test that ComputeWeightsInBins can handle SMALL matrix B
  # setup 
  # B matrix used: 
  #   1 2 2 2 3
  #   2 1 3 3 4 
  #   2 3 1 3 4
  #   2 3 3 1 2
  #   3 4 4 2 1
  
  B <- matrix(c(1,2,2,2,3,2,1,3,3,4,2,3,1,3,4,2,3,3,1,2,3,4,4,2,1), 5,5)
  vertex.weights <- c(1,0,0,1,0)
  adjusted.vertex.weights <- vertex.weights - mean(vertex.weights)
  nvertices <- length(vertex.weights)
  res <- SANTA:::ComputeWeightsInBins(B, adjusted.vertex.weights, nvertices)
  checkEquals(round(as.vector(res), 10), 
  	c(0.6, -0.4, -0.4, 0.6, -0.4, -0.2, 0.6, 0.6, 0.2, 0.6, -0.4, 0.2, 0.2, -0.8, 0.6, 0.0, -0.4, -0.4, 0.0, -0.8)) 
 
  # test that ComputeWeightsInBins can handle LARGE matrix B
  # setup
  # B matrix used: 
  #  1 1 1 1 ..
  #  2 2 2 2 ..
  #  3 3 3 3 ..
  #  4 4 4 4 ..
  #  : : : :
  nvertices <- 10000
  B <- matrix(c(rep(1:nvertices, nvertices-1), rep(1, nvertices)), nvertices, nvertices)
  vertex.weights <- rep(c(0,1), nvertices/2)
  adjusted.vertex.weights <- vertex.weights - mean(vertex.weights)
  res <- SANTA:::ComputeWeightsInBins(B, adjusted.vertex.weights, nvertices)
  checkEquals(round(as.vector(res[1:3, 1:3]), 10), 
  	c(0, 0.5, 0.5, 0, -0.5, 0, 0, 0, -0.5))
}

---
title: "Introduction to SANTA"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to SANTA}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{setup, echo=FALSE}
options(width = 75)
options(useFancyQuotes=FALSE)

```
```{hook-printfun, echo=FALSE}
library(knitr, quietly=TRUE)
library(formatR, quietly=TRUE)
knit_hooks$set(printfun = function(before, options, envir) {
    if (before) return()
    txt = capture.output(dump(options$printfun, '', envir = envir))
    ## reformat if tidy=TRUE
    if (options$tidy)
        txt = tidy.source(text=txt, output=FALSE,
                          width.cutoff=30L, keep.comment=TRUE,
                          keep.blank.line=FALSE)$text.tidy
    paste(c('\n```r\n', txt, '\n```\n'), collapse="\n")
    })
```

# Vignette for SANTA

## Alex J. Cornish and Florian Markowetz

### Introduction

This document contains instructions on how to employ the functions contained within the SANTA package. SANTA builds upon Ripley's K-function, a well-established approach in spatial statistics that measures the clustering of points on a plane, by applying it to networks. Using this approach, the `Knet`  and `Knode` functions are defined. `Knet` detects the clustering of hits on a network in order to measure the strength of association between the network and a phenotype. `Knode` ranks genes according their distance from the hits, providing a natural way of prioritising vertices for follow-up analyses.

SANTA addresses two complementary goals: it can (i) functionally annotate experimentally derived networks using curated gene sets, and (ii) annotate experimentally derived gene sets using curated networks.

The SANTA package also contains functions that can be used to measure the distance between vertex pairs (`GraphDiffusion` and `GraphMFPT`) and a function that can be used to visualise the `Knet` function (`plot.Knet`). The `igraph} package is used to handle the networks.

This vignette, along with the data provided in the SANTA package, are designed to allow the user to reproduce the results shown in the paper: Cornish, A. and Markowetz, F. (2014) *SANTA: quantifying the functional content of molecular networks.* PLOS Computational Biology. 10:9, e1003808. To reduce the time required to run the vignette, many of the case studies are set up to run with a reduced number of trials, permutations or gene sets. It is noted where and how the case studies differ from the paper analyses. The methods used by the SANTA package are further explained in the paper. 



### Overview of SANTA

##### The guilt-by-association principle

The guilt-by-association (GBA) principle states that genes that interact are more likely to share biological function. For example, two genes seen to interact in a synthetic genetic array (SGA) are more likely to encode products involved in the same pathway or complex than two genes not seen to interact. `Knet` uses this principle to measure the strength of association between a network and a phenotype. By measuring the clustering of the gene set across biological networks it becomes possible to identify the network that best explains the mechanisms and processes that underlie the set. This principle is also used by `Knode` to prioritise genes for follow up studies. If a set of genes has been identified as being associated with a particular cellular function, then the GBA principle states that genes seen interacting with this set are also likely to be involved in the function. 




##### The `Knet` function

The `Knet` function measures the strength of association between a network and a phenotype by quantifying the clustering of high-weight vertices on the network. High-weight vertices represent those genes that are part of a gene set or are most strongly linked with the phenotype. The GBA principle implies that the stronger the clustering, the better the network is at explaining the mechanisms that produce and maintain the phenotype.

The function is based upon Ripley's K-statistic, a tool from spatial statistics that measures the clustering of points on a plane. Ripley's K-statistic works by counting the number of points contained within circles of radius $s$ positioned around each point. The value of $s$ is increased and the area under the resulting curve (AUK) is measured. If points cluster, then they will be engulfed within the circles quicker, the function will increase faster and the AUK will be larger. By swapping the Euclidean distance across the plane with a measure of distance across the network, this statistic can be adapted for networks. We call this function the `Knet` function.

Unlike Ripley's K-statistic, the `Knet` function can measure the clustering of a set of vertices or a continuous distribution of vertex weights. If biological networks are being used, then a high weight would indicate that a gene is strongly linked with the phenotype being studied. In some scenarios, weights may not be available for all vertices. Vertices with missing weights can be included within the network. However, they should be given a weight of `NA} to ensure that their distribution does not affect the significance analysis.

The significance of clustering is quantified by permuting the vertex weights across the networks. Vertices with missing weights are excluded. The area under the `Knet` function for the observed set is then compared to the area under the `Knet` function for the permuted sets and a z-test is used to compute a p-value. The p-values produced represents the probability that a clustering of vertex weights at least as strong as that  observed occurs given that the vertex weights are distributed randomly across the network.

The `Knet` function is:

$$
K^{net}(s) = \frac{2}{(\bar{p} n)^{2}}\sum_{i}p_{i}\sum_{j}(p_{j} - \bar{p})~\textbf{I}(d^{g}(i,j)\leq s) 
$$

where $p_i$ is the phenotype observed at vertex $i$, $\textbf{I}(d^{g}(i,j)\leq s)$ is an identity function that equals $1$ if the distance between vertex $i$ and $j$ is less than $s$ and 0 otherwise, and $\bar{p} = \frac{1}{n}\sum_{i=1}^n p_i$.




##### The `Knode` function

Taking the inner sum of the `Knet` equation makes it possible to rank vertices by their distance from high-weight vertices. We call this function the `Knode` function. This function provides a natural way to prioritise genes for follow up studies.

The `Knode` function is:

$$
K_{i}^{node}(s) = \frac{2}{p}\sum_{j}(p_{j} - \bar{p})~\textbf{I}(d^{g}(i,j)\leq s) 
$$




##### Measuring the distance between pairs of vertices

SANTA contains three methods for measuring the distance between pairs of vertices. Each method incorporates different aspects of the network.

* Shortest paths. The distance along the shortest path connecting two vertices. In a graph without weighted edges, the length of the shortest path is equal to the number of edges in the path. In a graph with weighted edges, the length of the shortest path is equal to the sum of the edge weights in the path. This is the simplest measure of distance and is used by default by the SANTA package.
* Diffusion kernel-based method. The diffusion kernel-based distance measure can be considered as the physical process of diffusion though the graph. Therefore, unlike the shortest paths method, the diffusion kernel-based method incorporates not only the distance along the shortest path connecting two vertices, but the distances along multiple paths. It therefore incorporates more of the network structure. 
* Mean first-passage time-based (MFPT) method. The MFPT distance measure computes the distance between vertex $i$ and $j$ ($m_{i,j}$) by computing the expected amount of time that a random walk emanating from node $i$ takes to reach node $j$ for the first time. Much like the diffusion kernel-based method, the  MFPT method incorporates the number of paths connecting a pair of vertices.




### Case studies

##### i) `Knet` successfully identifies clustering on simulated networks

In this section, we will use simulated networks to demonstrate that the `Knet`-function successfully quantifies the clustering of hits on a network.

In order to demonstrate how the `Knet` function can be used to measure the clustering of hits on a network, we will simulate a network, apply clusters of hits of different clustering strengths to the network, and use the `Knet` function to quantify the strength of clustering. 

We will use the Barabasi-Albert model of preferential attachment to simulate the networks, as it has been observed that this methods produced networks with attributes similar to many real-world networks. A network of 500 vertices is both large enough to contain clusters of hits of different strengths and small enough to quickly run a large number of trials upon. In the Barabasi-Albert model, vertices are added stepwise and connected to previously-added vertices. This is done under a probability distribution based upon the degree of each vertex. If $power=1$, then the relationship between the probability distribution and vertex degree is linear. We will use this value, as it creates a network with multiple high-degree hubs, much like many biological networks. Each new vertex is connected to $m$ previously-added vertices. In order to create a moderately-dense network, we will set $m=1$. 

```{case_i_graph_create}
# generate the simulated network
require(SANTA, quietly=TRUE)
require(igraph, quietly=TRUE)
set.seed(1) # for reproducibility
g <- barabasi.game(n=500, power=1, m=1, directed=FALSE) 
```

A cluster of hits is simulated by first randomly choosing a seed vertex. All other vertices within the network are then ranked according to their distance to the from the seed vertex. The distance can be measured using any of the previously described distance metrics (shortest paths, diffusion kernel and MFPT). In this example, we will use the shortest paths metric, as it is the quickest to run. However, running the analysis with the other methods will produce the same results. 

```{case_i_compute_d}
# measure the distance between pairs of vertices in g
dist.method <- "shortest.paths"
D <- DistGraph(g, dist.method=dist.method, verbose=FALSE) 
```

The `Knet` function computes the distances between each pair of vertices in $g$. However, to reduce the time required to run the function, these distances can be computed outside the function. The `Knet` function cannot be run on the raw continuous distances. Therefore, it is necessary to place the pairs of vertices into discreet bins. This is done using the `BinGraph} function.  

```{case_i_compute_b}
# place the distances into discreet bins
B <- BinGraph(D, verbose=FALSE)
```

Next, we will generate clusters of hits and measure the clustering strength using the `Knet` function. The $s$ vertices positioned closest to the seed vertex form the sample set. From this set, 5 vertices are randomly selected to form the hit set. The size of the hit set needs to be small compared to the number of vertices in the network. Increasing the value of $s$ decreases the strength of the clustering. We will test 5 different values of $s$ in order to simulate different clustering strengths. A value of $s$ equal to the total number of vertices in the network produces a hit set containing vertices randomly selected from the entire network. For each value of $s$, we will conduct 10 trials. In the associated paper, 1000 trials were run. Here we will also use 100 permutations, rather than 1000, in order to reduce the time required.

```{case_i_trials}
cluster.size <- 5
s.to.use <- c(10, 20, 50, 100, 500)
n.trials <- 10
pvalues <- array(0, dim=c(n.trials, length(s.to.use)), 
    dimnames=list(NULL, as.character(s.to.use)))

# run the trials for each value of s
for (s in s.to.use) {    	
    for (i in 1:n.trials) {
        # generate the hit set
        seed.vertex <- sample(vcount(g), 1) # identify seed
        sample.set <- order(D[seed.vertex, ])[1:s]
        hit.set <- sample(sample.set, cluster.size)
        
        # measure the stength of association
        g <- set.vertex.attribute(g, name="hits", 
            value=as.numeric(1:vcount(g) %in% hit.set))
        pvalues[i, as.character(s)] <- Knet(g, nperm=100, 
            dist.method=dist.method, vertex.attr="hits", 
            B=B, verbose=FALSE)$pval
    }
}
```

Figure 1 contains the results of the simulation study. The `Knet` function correctly identifies sets of hits created using smaller values of $s$ as clustering more significantly. 

```{case_i_plot, results="asis", echo=FALSE}
boxplot(-log10(pvalues), xlab="cutoff", ylab="-log10(p-value)")
```

```{case_i_cleanup, echo=FALSE}
# cleanup
rm(B, D, g)
```




##### ii) Comparing `Knet` to Compactness

Nexy we will demonstrate the effectiveness of the `Knet` function by comparing it to a simpler metric.

Glaab et al. define the compactness score as the mean distance between pairs of hits in a network. By permuting the vertices belonging to the hit set and comparing the observed compactness score to compactness score of the permuted sets, an empirical p-value can be produced. This is much like `Knet`. However, unlike `Knet`~, the adapted compactness score cannot be applied to continuous distributions of vertex weights and does not incorporate vertex degree when measuring the significance of association. This is demonstrated in this toy example.

First, we create the network. The vertices in this network have a range of degrees, allowing us to demonstrate the differences between `Knet` and `Compactness`. We will apply 2 sets of vertex weights, each of which will contain two interacting hits.

```{case_ii_create_network}
# create the network
n.nodes <- 12
edges <- c(1,2, 1,3, 1,4, 2,3, 2,4, 3,4, 1,5, 5,6, 
           2,7, 7,8, 4,9, 9,10, 3,11, 11,12)
weights1 <- weights2 <- rep(0, n.nodes)
weights1[c(1,2)] <- 1
weights2[c(5,6)] <- 1 

g <- graph.empty(n.nodes, directed=FALSE)
g <- add.edges(g, edges)
g <- set.vertex.attribute(g, "weights1", value=weights1)
g <- set.vertex.attribute(g, "weights2", value=weights2)
```

```{case_ii_create_plot, results="asis", echo=FALSE}
par(mfrow=c(1,2))

colors <- rep("grey", n.nodes)
colors[which(weights1 == 1)] <- "red"
g <- set.vertex.attribute(g, "color", value=colors)
plot(g)

colors <- rep("grey", n.nodes)
colors[which(weights2 == 1)] <- "red"
g <- set.vertex.attribute(g, "color", value=colors)
plot(g)

par(mfrow=c(1,1))
g <- remove.vertex.attribute(g, "color")
```

Figure 2 shows the location of each set of hits on the network. The distance between the hits in set 1 and set 2 are the same. However, intuitively we expect that the clustering of the hits in set 2 to be more significant, as the hits have fewer interacting partners and are therefore less likely to be observed interacting by chance.

We will now apply `Knet` and `Compactness` to each set of hits in order to measure the clustering significance. We will run the function with 100 permutations. Increasing the number of permutations increases the accuracy of the functions. However, it also increases the run time. 1000 permutations were used to create the results shown in the paper. The shortest paths distance measure will be used. However, other methods will produce the same results.

```{case_ii_pvalues}
# set 1
Knet(g, nperm=100, vertex.attr="weights1", verbose=FALSE)$pval
Compactness(g, nperm=100, vertex.attr="weights1", verbose=FALSE)$pval

# set 2
Knet(g, nperm=100, vertex.attr="weights2", verbose=FALSE)$pval
Compactness(g, nperm=100, vertex.attr="weights2", verbose=FALSE)$pval
```

The `Compactness` function does not incorporate vertex degree when measuring the significance of hit clustering. For this reason, it outputs a similar p-value for each hit set. `Knet` incorporates the global distribution of vertices and therefore identifies set 2 as clustering more significantly than set 1. It is therefore preferable to use `Knet` instead of `Compactness` when the degree of vertices varies across a network.





##### iii) Using `Knet` to demonstrate that correlation in GI profile produce functionally more informative networks

In this section we will demonstrate how the `Knet` function can be used to identify the most effective way of building an interaction network.

There is evidence that protein function is more closely related to global similarity between genetic interaction (GI) profile than to individual interactions. In order to test this, we will use the `Knet` function to quantify the strength of association between GO terms and two S. cerevisiae networks: a network of raw interactions and a network created from correlations in interaction profile. Both of these will be created using the data produced by Costanzo et al. If the GO terms associate more strongly with a network, then this is indication that the network is more functionally informative.

First, we will load two pre-processed networks, in the form of `igraph` objects from the `SANTA` package. `SANTA` requires all networks to be in the form of `igraph` objects. The data used to create these networks is available from http://drygin.ccbr.utoronto.ca/~costanzo2009/ and the method used is described in the associated SANTA paper.

`Knet` is also able to incorporate different edge distances when quantifying the clustering of high-weight vertices. Edge distances can represent different biological properties, such as the strength of a physical interaction between 2 gene products, or in the case of our 2 networks, the strength of the genetic interaction. The smaller the edge distance, the stronger or more significant the interaction. When creating the network, we converted the correlation coefficients into edge distances by taking the $-log10$ of the scores. These distances have been stored under the edge attribute `distance`.

```{case_iii_load_networks}
# load igraph objects
data(g.costanzo.raw)
data(g.costanzo.cor)
networks <- list(raw=g.costanzo.raw, cor=g.costanzo.cor)
network.names <- names(networks)
network.genes <- V(networks$raw)$name 
# genes identical across networks
```
 
Associations between genes and GO terms will be obtained from the Gene Ontology (GO) project using the `org.Sc.sgd.db` package. In order to reduce the time required to run this analysis, we will use only 5 GO terms. However, it is easy to run this analysis with many more GO terms, as done in the associated paper. 

```{case_iii_go}
# obtain the GO term associations from org.Sc.sgd.db package
library(org.Sc.sgd.db, quietly=TRUE)
xx <- as.list(org.Sc.sgdGO2ALLORFS)
go.terms <- c("GO:0000082", "GO:0003682", "GO:0007265", 
              "GO:0040008", "GO:0090329")
# apply the GO terms to the networks
for (name in network.names) {
    for (go.term in go.terms) {
        networks[[name]] <- set.vertex.attribute(
            networks[[name]], name=go.term, 
            value=as.numeric(network.genes %in% xx[[go.term]]))
    }
} 
```

We will now apply the `Knet` function to each GO terms on each network in order to quantify the strength of association. If a gene is associated with the GO term it will given a weight of 1. If it is not associated, it will be given a weight of 0. We will now store these scores in the 2 graphs under a vertex attribute called `rdds`. Due to the time required to compute the distance matrices, the number of permutations has been reduced to 10. Running at least 1000 permutations is recommended.

```{case_iii_association}
results <- list()
for (name in network.names) {
    results[[name]] <- Knet(networks[[name]], nperm=10, 
        vertex.attr=go.terms, edge.attr="distance", verbose=FALSE)
    results[[name]] <- sapply(results[[name]], 
        function(res) res$pval)
}
```

```{case_iii_cleanup, echo=FALSE}
# cleanup
rm(g.costanzo.cor, g.costanzo.raw, network.genes, networks, xx)
```





##### iv) Using `Knet` to investigate the functional rewiring of the yeast interaction network in response to MMS-treatment

In this section we will demonstrate how the `Knet`-function can be used to investigate the functional rewiring of a cell by comparing the strength of association of gene sets between 2 networks.

Bandyopadhyay et al. mapped genetic interaction (GI) networks in yeast under normal laboratory conditions and in yeast exposed to the DNA-damaging agent methyl methanesulfonate (MMS). In order to investigate the differences between these functional networks, we will use the `Knet`-function to quantify the strength of association of a set of genes involved in responding to DNA damage. While we cannot learn much from this example, sets of genes involved in other processes can also be tested in order to identify other processes involved in the response to DNA damage.

We will now load the two interactions networks. These networks were created using correlations in GI profile, as described in the previous section. This is because, as previously demonstrated, using correlations in GI profile as edges results in a functionally more-informative network.

```{case_iv_load_igraph}
# load igraph objects
data(g.bandyopadhyay.treated)
data(g.bandyopadhyay.untreated)
networks <- list(
    treated=g.bandyopadhyay.treated, 
    untreated=g.bandyopadhyay.untreated
)
network.names <- names(networks)
```

We will the Gene Ontology (GO) project to identify those genes involved in responding to DNA damage. By changing the code below, it is possible to measure the strength of association of gene sets involved in different functions. In the associated paper, we measured the strength of association of 194 GO terms with each network. 

```{case_iv_go}
# obtain GO term associations
library(org.Sc.sgd.db, quietly=TRUE)
xx <- as.list(org.Sc.sgdGO2ALLORFS)
# change to use alternative GO terms
associated.genes <- xx[["GO:0006974"]] 
associations <- sapply(networks, function(g) 
    as.numeric(V(g)$name %in% associated.genes), 
    simplify=FALSE)
networks <- sapply(network.names, function(name) 
    set.vertex.attribute(networks[[name]], "rdds", 
    value=associations[[name]]), simplify=FALSE)
```

We will now apply the `Knet` function to the GO term on each network in order to measure the strength of association. The greater the number of permutations run, the greater the accuracy of the p-value.

We will also run `Knet` using the shortest paths method to compute the distance between vertex pairs. However, as shown in the associated paper, the `diffusion` and `mfpt` distance measures produce the same results. This can be demonstrated by replacing the `dist.method` argument below with one of the alternative distance measures. By running this analysis with additional GO terms, the robustness of the `Knet` across different distance measures can be seen. Due to the time required to compute the distance matrices, the number of permutations has been reduced to 10. Running with least 1000 permutations is recommended.

```{case_iv_association}
results <- sapply(networks, function(g) Knet(g, nperm=10, 
    dist.method="shortest.paths", vertex.attr="rdds", 
    edge.attr="distance"), simplify=FALSE)
p.values <- sapply(results, function(res) res$pval)
p.values
```

Since the GO term tested was `response to DNA damage stimulus`, it is not surprising that the gene set associated more significantly with the treated network (as shown by the lower p-value). The yeast exposed to the DNA-damaging agent have activated or upregulated pathways involved in responding to the agent, thereby increasing the strength of the genetic interaction between DNA-damage response-related genes.

```{case_iv_cleanup, echo=FALSE}
# cleanup
rm(xx, networks, g.bandyopadhyay.treated, g.bandyopadhyay.untreated, associated.genes)
```




##### v) Using `Knet` to investigate the functional rewiring of the yeast interaction network in response to UV damage

In this section, we will conduct similar analysis to the previous section. However, in this section, we will use a network created from GIs mapped in S. cerevisiae exposed to a high dosage of UV and GIs mapped from untreated S. cerevisiae. Due to the reduced number of interactions mapped in this study, it is not possible to build networks from correlations in GI. Therefore, the networks are built from the raw interactions. 

```{case_v_load}
# laod igraph object
data(g.srivas.high)
data(g.srivas.untreated)
networks <- list(
    high=g.srivas.high, 
    untreated=g.srivas.untreated
)
network.names <- names(networks)
```

We will again test only a single GO term (GO:0000725, recombinational repair). In the associated paper, a greater number of GO terms are tested.

```{case_v_go}
# obtain GO term associations
library(org.Sc.sgd.db, quietly=TRUE)
xx <- as.list(org.Sc.sgdGO2ALLORFS)
associated.genes <- xx[["GO:0000725"]] 
associations <- sapply(networks, function(g) 
    as.numeric(V(g)$name %in% associated.genes), 
    simplify=FALSE)
networks <- sapply(network.names, function(name)
    set.vertex.attribute(networks[[name]], "dsbr", 
    value=associations[[name]]), simplify=FALSE)
```

The `Knet` function will again be used to compare the strength of association of the GO term with the 2 networks. Due to the time required to compute the distance matrices, the number of permutations has been reduced to 10. Running with least 1000 permutations is recommended.

```{case_v_association}
p.values <- sapply(networks, function(g) 
       Knet(g, nperm=10, dist.method="shortest.paths", 
       vertex.attr="dsbr", edge.attr="distance")$pval)
p.values
```

As expected, the GO term associates more strongly with the UV treated network. This is likely due to the functional rewiring that occurs within the yeast when exposed to DNA-damaging UV radiation.

```{case_v_cleanup, echo=FALSE}
# cleanup
rm(xx, networks, g.srivas.high, g.srivas.untreated)
```




##### vi) Using `Knet` to identify the network most informative about cancer cell lines

In this section, we will apply a continuous distribution of vertex weights, describing the strength of evidence that a gene is involved in the maintenance of cancer cell lines, to two networks in order to identify the network that best explains the mechanisms that underlie cell line maintenance.

Biological networks can be built using a wide range of data sources, including co-expression data, data on physical interactions or genetic interaction data. Networks can also be created by combining data sources. The HumanNet network was created by combining 21 data sources. The IntAct network is created using curated data on physical interactions.

In the next section we will apply pre-processed RNAi data from cancer cell lines to both networks as vertex weights. The GENE-E program has been used to convert the essentiality scores for each shRNA into gene-wise p-values. This p-values are contained within the loaded dataset. We will also convert these p-values into vertex weights that can be applied to the networks. By taking the $-log_{10}$ of the p-values, we ensure that the most essential genes are given the highest vertex weights. 

```{case_vi_rnai}
# import and convert RNAi data
data(rnai.cheung)
rnai.cheung <- -log10(rnai.cheung)
rnai.cheung[!is.finite(rnai.cheung)] <- max(rnai.cheung[is.finite(rnai.cheung)])
rnai.cheung[rnai.cheung < 0] <- 0
```

Next, we will load the interaction data for the two network and create an `igraph` object for each. No edge weights are given for the IntAct network. Each of the interactions in the HumanNet network is associated with a log likelihood score (LLS), describing the probability the functional interaction occurring given the data used. In order to reduce the density of the network, interactions with an $LLS < 2$ have been removed. Distances for the remaining edges have been produced by dividing the weights by the maximum weight and taking the $-log_{10}$ of this score. We have removed all genes not part of the largest cluster from each network. 

```{case_vi_networks}
# import and create IntAct network
data(edgelist.intact)
g.intact <- graph.edgelist(as.matrix(edgelist.intact), 
    directed=FALSE)

# import data and create HumanNet network
data(edgelist.humannet)
g.humannet <- graph.edgelist(as.matrix(edgelist.humannet)[,1:2], 
    directed=FALSE)
g.humannet <- set.edge.attribute(g.humannet, "distance", 
    value=edgelist.humannet$distance)
networks <- list(intact=g.intact, humannet=g.humannet)
```

Next, we will apply the vertex weights to the networks. Not all networks genes have vertex weights available.

```{case_vi_weights}
network.names <- names(networks)
network.genes <- sapply(networks, get.vertex.attribute, 
    name="name", simplify=FALSE)
rnai.cheung.genes <- rownames(rnai.cheung)
cancers <- colnames(rnai.cheung)

for (cancer in cancers) {
    for (name in network.names) {
        vertex.weights <-rep(NA, vcount(networks[[name]]))
        names(vertex.weights) <- network.genes[[name]]
        common.genes <- rnai.cheung.genes[rnai.cheung.genes
            %in% network.genes[[name]]] 
        vertex.weights[common.genes] <- rnai.cheung[common.genes, cancer]
        networks[[name]] <- set.vertex.attribute(networks[[name]], 
            cancer, value=vertex.weights)
    }
}
```

We will now apply the `Knet`-function to these scores. Due to the time required to compute the distance matrices, the number of permutations has been reduced to 10. Running with least 1000 permutations is recommended.

```{case_vi_association}
knet.res <- sapply(networks, Knet, nperm=10, 
   dist.method="shortest.paths", vertex.attr=cancers, 
   edge.attr="distance", simplify=FALSE)
p.values <- sapply(knet.res, function(i) sapply(i, 
   function(j) j$pval))
p.values
```

The RNAi data from the colon and ovarian cancer cell lines associate strongly with both networks. However, they associate associate more strongly with the HumanNet network, indicating that the HumanNet network better explains the cellular mechanisms involved in maintaining these cell lines.

```{case_vi_cleanup, echo=FALSE}
# cleanup
rm(rnai.cheung, rnai.cheung.genes, networks, network.genes, edgelist.humannet, edgelist.intact, common.genes, g.humannet, g.intact)
```


### Session info

This document was produced using:

```{session_info}
sessionInfo()
```

\name{GraphMFPT}
\alias{GraphMFPT}
\title{
Compute mean first-passage time-based distance matrix
}
\description{
Using the mean first-passage time method, compute the distances between vertex pairs in an undirected graph, with or without edge weights.
}
\usage{
GraphMFPT(g, v=V(g), edge.attr.weight=NULL, average.distances=TRUE)
}
\arguments{
  \item{g}{
\code{igraph} object, the graph to work on. 
}
  \item{v}{
\code{igraph} object or numeric vector, the vertices from which each distance is calculated.
}
  \item{edge.attr.weight}{
String, the name of the edge attribute to be used as weights along the edges. Greater weights indicate a stronger interaction between the two genes (this is the opposite to edge distances, where smaller distances indicate stronger interactions). If \code{NULL} then each edge is assumed to have a weight of 1.   
}
  \item{average.distances}{
Logical, if \code{TRUE} then the distance from vertex A to B and the distance from vertex B to A are averaged to give a single distance. Otherwise, two different distances may be returned.
}
}
\details{
The mean first-passage time from vertex A to vertex B is defined as the expected number of steps taken on a random walk emanating from vertex A until the first arrival at vertex B. This provides a method of measuring the distance between pairs of vertices that does not simply take into account the distance along the shortest path, but rather incorporates how well the two vertices are connected across multiple paths.

The mean first-passage time from vertex A to vertex B is not necessarily the same as the mean first-passage time from vertex B to vertex A. If a symmetric distance matrix is required, reciprocal distances can be averaged to give a single value for each vertex pair.

If a vertex pair is unconnected, then the distance between the vertices is \code{Inf}.

The distance from vertex A to vertex A is always 0.
}
\value{
Numeric matrix, containing the mean first-passage time-derived vertex pair distance between each vertex in \code{v} and every vertex in \code{g}.
}
\references{
White, S. and Smyth, P. (2003). \emph{Algorithms for Estimating Relative Importance in Networks}. Technical Report UCI-ICS 04-25.
}
\author{
Alex J. Cornish \email{a.cornish12@imperial.ac.uk}
}
\seealso{
\code{\link[SANTA:GraphDiffusion]{GraphDiffusion}},
\code{\link[igraph]{shortest.paths}}
}
\examples{
# create a  and compute the mean first-passage time-based vertex pair distance matrix
g <- erdos.renyi.game(6, p.or.m=0.5, directed=FALSE)
GraphMFPT(g) 
plot(g, layout=layout.fruchterman.reingold)
}

\name{Knet}
\alias{Knet}
\title{
Measure the strength of association between a phenotype and a network by computing the strength of hit clustering on the network
}
\description{
Compute the strength of clustering of high-weight vertices (hits) on a network using a modified version of Ripley's K-statistic. This method can be used to measure the strength of association between a phenotype or function and a network. 
}
\usage{
Knet(g, nperm=100, dist.method=c("shortest.paths", "diffusion", "mfpt"),
vertex.attr="pheno", edge.attr=NULL, correct.factor=1, nsteps=1000, 
prob=c(0, 0.05, 0.5, 0.95, 1), B=NULL, verbose=TRUE)
}
\arguments{
  \item{g}{
\code{igraph} object, the network to work on. 
}
  \item{nperm}{
Integer value, the number of permutations to be completed.
}
  \item{dist.method}{
String, the method used to calculate the distance between vertex pairs.
}
  \item{vertex.attr}{
Character vector, the name of the vertex attributes under which the vertex weights to be tested are stored. The vector can contain one or more elements.
}
  \item{edge.attr}{
String, the name of the edge attribute to be used as distances along the edges. If an edge attribute with this name is not found, then each edge is assumed to have a distance of 1. 
}
\item{correct.factor}{
Numeric value, if the network contains unconnected vertices, then the distance between these vertices is set as the maximum distance between the connected vertices multiplied by \code{correct.factor}.
}
  \item{nsteps}{
Integer value, the number of bins into which vertex pairs are placed.
}
  \item{prob}{
Numeric vector, the quantiles to be calculated for the \code{Knet} permutations.
}
  \item{B}{
Symmetrical numeric matrix. A precomputed distance bin matrix for \code{g} output by the \code{BinGraph} function. If \code{NULL}, then B is computed within the \code{Knet} function. 
}
  \item{verbose}{
Logical, if \code{TRUE} messages about the progress of the function are displayed. 
}
}
\details{
The SANTA method uses the 'guilt-by-association' principle to measure the strength of association between a network and a phenotype. It does this by measuring the strength of clustering of the phenotype scores across the network. The stronger the clustering, the greater the association between the network and the phenotype. 

The SANTA method applies Ripley's K-function, a well-established approach to spatial statistics that measures the strength of clustering of points on a plane, and extends it in a number of ways. First, a Knet function is defined by adapting the approach for networks using vertex pair distance measures. Second, vertex weights are incorporated into Knet and the importance of vertices made relative to their own associated weight. Third, the mean vertex weight is subtracted from each individual vertex weight when calculating the Knet function. This means that the Knet function measures the degree of vertex weight clustering relative to a random distribution of vertex weights. The Knet function is defined as 

\deqn{K^{net}[s]=\frac{2}{p^2} \sum_i p_i \sum_j(p_j - \bar{p}) I(d^g[i,j]<=s)}{K^{net}[s] = 2/p^2 * sum_i{(p_i) sum_j{(pj - bar{p}) I(dg(i,j)<=s)}}}

where \eqn{p_i} is the weight of vertex \eqn{i}, \eqn{\bar{p}} is the mean vertex weight across all vertices, and \eqn{I(dg[i,j]<=s)} is an identity function, equaling 1 if vertex \eqn{i} and vertex \eqn{j} are within distance \eqn{s} and \eqn{0} otherwise. 

In order to derive a p-value and quantify the significance of the observed distribution of weights, the observed Knet-curve is compared to Knet-curves obtained using the same network but randomly permuted vertex weights. Vertices with missing weights (\code{NA}) are not included within these permutations. The area under the Knet-curve (AUK) is calculated for the observed network and each of the permuted networks and a z-score used to produce a p-value. This p-value indicates the probability an observed AUK at least this high is seen given the null hypothesis that the vertex weights are randomly distributed.

Vertex weights should be greater or equal that zero or equal to NA if the weight is missing.
}
\value{
If one vertex attribute is input, \code{Knet} is run on the single set of vertex weights and a list containing the statistics below is returned. If more than one vertex attribute is input, then \code{Knet} is run on each set of vertex weights and a list containing an element for each vertex attribute is returned. Each element contains a sub-list containing the statistics below for the relevant vertex attribute.

\item{K.obs}{Knet-function curve for the observed vertex weights.}
\item{AUK.obs}{Area under the Knet-function curve (AUK) for the observed vertex weights.}
\item{K.perm}{Knet-function curve for each permutation of vertex weights. Equals \code{NA} if no permutations are completed.}
\item{AUK.perm}{Area under the Knet-function curve (AUK) for each permutation of vertex weights. \code{NA} if no permutations are completed.}
\item{K.quan}{Quantiles for the permuted Knet-function curves. \code{NA} if no permutations are completed.}
\item{pval}{p-value, calculated from a z-score derived from the observed and permuted AUKs. \code{NA} if no permutations are completed.}
}
\references{
Cornish, A.J. and Markowetz, F. (2014) \emph{SANTA: Quantifying the Functional Content of Molecular Networks.}. PLOS Computational Biology. 10:9, e1003808.

Okabe, A. and Yamada, I. (2001). \emph{The K-function method on a network and its computational implementation} Geographical Analysis. 33(3): 271-290.
}
\author{
Alex J. Cornish \email{a.cornish12@imperial.ac.uk} and Florian Markowetz
}
\seealso{
\code{\link[SANTA:Knode]{Knode}}
}
\examples{
# apply Knet to a network with hit clustering
g.clustered <- barabasi.game(50, directed=FALSE)
g.clustered <- SpreadHits(g.clustered, h=10, lambda=10)
res.clustered <- Knet(g.clustered, nperm=100, vertex.attr="hits")
res.clustered$pval
plot.Knet(res.clustered)

# apply Knet to a network without hit clustering
g.unclustered <- barabasi.game(50, directed=FALSE)
g.unclustered <- SpreadHits(g.unclustered, h=10, lambda=0)
res.unclustered <- Knet(g.unclustered, nperm=100, vertex.attr="hits")
res.unclustered$pval
plot.Knet(res.unclustered)
}

\name{BinGraph}
\alias{BinGraph}
\title{
Compute a distance bin matrix from a distance matrix
}
\description{
In order for the \code{Knet} or \code{Knode} functions to be run on a network, it is necessary to place the raw distances between each vertex pair into discreet distance bins, as the functions are unable to handle continuous distributions of distances. This is done automatically within the \code{Knet} and \code{Knode} functions, but can also be done separately using \code{BinGraph}. 
}
\usage{
BinGraph(D, nsteps=1000, equal.bin.fill=TRUE, verbose=TRUE)
}
\arguments{
  \item{D}{
Numeric matrix, a distance matrix output by \code{DistGraph}.
}
  \item{nsteps}{
Integer value, the desired number of bins across which the distances are to be split. If there are too few unique distances to fill each bin, then fewer bins are returned. 
}
  \item{equal.bin.fill}{
Logical, if \code{TRUE} then the function attempts to fill each bin with an equal number of vertex pairs.
}
  \item{verbose}{
Logical, if \code{TRUE} messages about the progress of the function are displayed. 
}
}
\details{
In order for the \code{Knet} or \code{Knode} functions to be run, the vertex pair distances (as computed by \code{DistGraph}) but be split into bins. This is done as part of the \code{Knet} or \code{Knode}. However, this step is often slow for large networks and therefore the \code{BinGraph} function is provided separately, in order to avoid repeat computation. 

Each vertex pair is placed into a bin, either ranging from 1 to \code{nsteps}, or from 1 to the number of unique distances. 

If \code{equal.bin.fill} is \code{FALSE}, then the bin each vertex pair is placed into is directly proportional to the largest vertex pair distance. For example, if the distance between the pair is 25\% of the largest distance and \code{nsteps} equals \code{100}, then the vertex pair will be placed into bin \code{25}. However, this can create problems when there are a small number of edges with especially large distances, as this can result in the majority of vertex pairs being placed into a small number of bins. This can reduce the effectiveness of the \code{Knet} and \code{Knode} functions. Therefore, when \code{equal.bin.fill} is \code{TRUE}, the function attempts to fill each bin with an equal number of vertex pairs. If there are a large number of tied distances, then the bins may not be filled equally. 
}
\value{
Integer matrix with the same dimensions as \code{D}.
}
\author{
Alex J. Cornish \email{a.cornish12@imperial.ac.uk}
}
\seealso{
\code{\link[SANTA:DistGraph]{DistGraph}}
}
\examples{
# create network and calculate the distance matrix using the shortest paths measure
g1 <- barabasi.game(6, directed=FALSE)
plot(g1, layout=layout.fruchterman.reingold)
D1 <- DistGraph(g1, dist.method="shortest.paths")
# place the distances into distance bins
BinGraph(D1, nsteps=100)

# create network and calculate the distance matrix using diffusion kernel-based measure
g2 <- erdos.renyi.game(6, p.or.m=0.5, directed=FALSE)
g2 <- set.edge.attribute(g2, name="distance", value=runif(ecount(g2)))
plot(g2, layout=layout.fruchterman.reingold)
# place the distances into distance bins
D2 <- DistGraph(g2, dist.method="diffusion", edge.attr="distance")
BinGraph(D2, nsteps=100)
}

\name{SANTA.data}
\alias{edgelist.humannet}
\alias{edgelist.intact}
\alias{g.bandyopadhyay.treated}
\alias{g.bandyopadhyay.untreated}
\alias{g.costanzo.cor}
\alias{g.costanzo.raw}
\alias{g.srivas.high}
\alias{g.srivas.untreated}
\alias{go.entrez}
\alias{rnai.cheung}
\title{
Pre-processed dataset for the SANTA vignette
}
\description{
Pre-processed network and expression data for use in the accompanying vignette. 
}
\details{
\describe{
  \item{edgelist.humannet}{Data frame of functional interactions from the HumanNet network with edge distances derived from the accompanying log-likelihood scores.}
  \item{edgelist.intact}{Data frame of \emph{H. sapiens} physical interactions from the IntAct database downloaded on 2013-05-02.}
  \item{g.bandyopadhyay.treated}{\emph{S. cerevisiae} interaction network created from the MMS-treated GI data from the study by Bandyopadhyay et al. Interactions represent correlation in GI profile. Network is an \code{igraph} object.}
  \item{g.bandyopadhyay.untreated}{\emph{S. cerevisiae} interaction network created from the untreated GI data from the study by Bandyopadhyay et al. Interactions represent correlation in GI profile. Network is an \code{igraph} object.}
  \item{g.costanzo.cor}{\emph{S. cerevisiae} interaction network created from the GI data from the study by Costanzo et al. Interactions represent correlation in GI profile. Network is an \code{igraph} object.}
  \item{g.costanzo.raw}{\emph{S. cerevisiae} interaction network created from the GI data from the study by Costanzo et al. Interactions represent raw GIs. Network is an \code{igraph} object.}
  \item{g.srivas.high}{\emph{S. cerevisiae} interaction network created from the high UV-dosage GI data from the study by Srivas et al. Interactions represent raw GIs. Network is an \code{igraph} object.}
  \item{g.srivas.untreated}{\emph{S. cerevisiae} interaction network created from the untreated GI data from the study by Srivas et al. Interactions represent raw GIs. Network is an \code{igraph} object.}
  \item{go.entrez}{Genes associated with the GO term GO:0042981 (regulation of apoptotic process).}
  \item{rnai.cheung}{Matrix of gene-wise essentiality p-values for 6 cancer cell lines. Rows represent genes and columns represent cancers. This data was created from the RNAi screens conducted by Cheung et al. The weight of evidence approach was used to compute essentiality scores for each shRNA and GENE-E was used to collapse the shRNA scores into gene-wise scores.}
}
}
\references{
Lee, I., Blom, U.M., Wang, P.I. et al. (2011). \emph{Prioritizing candidate disease genes by network-based boosting of genome-wide association data.} Genome Research. 21, 1109-21. 

Orchard, S., Ammari, M., Aranda, B. et al. (2014). \emph{The MIntAct project - IntAct as a common curation platform for 11 molecular interaction databases.} Nucleic Acids Research. 42:1, D358-63.

Costanzo, M., Baryshnikova., A., Bellay, J. et al. (2010). \emph{The Genetic Landscape of a Cell.} Science. 327:5964, 425-31. 

Bandyopadhyay, S., Mehta, M., Kuo, D. et al. (2010) \emph{Rewiring of Genetic Networks in Response to DNA Damage.} Science. 330, 1385-90.

Srivas, R., Costelloe, T., Carvunis, A. et al. (2013) \emph{A UV-induced genetic network links the RSC complex to nucleotide excision repair and shows dose-dependant rewiring.} Cell Reports. 5:6, 1714-24.

Ashburner, M., Ball, C.A., Blake, J.A. et al. (2000) \emph{Gene Ontology: tool for the unification of biology.} Nature Genetics. 25, 25-9.

Cheung, H.W, Cowley, G.S., Weir, B.A. et al. (2011) \emph{Systematic investigation of genetic vulnerabilities across cancer cell lines reveals lineage-specific dependencies in ovarian cancer.} PNAS. 108:30, 12372-7.
}
\examples{
data(g.bandyopadhyay.treated)
}

\name{Knode}
\alias{Knode}
\title{
Rank vertices by their strength of association with high-weight vertices 
}
\description{
Rank vertices by their strength of association with high-weight vertices using a modified version of Ripley's K-statistic. Vertex weights can either be binary or positive and continuous.
}
\usage{
Knode(g, dist.method=c("shortest.paths", "diffusion", "mfpt"), vertex.attr="pheno", 
edge.attr=NULL, correct.factor=1, nsteps=1000, B=NULL, verbose=TRUE)
}
\arguments{
  \item{g}{
\code{igraph} object, the network to work on. 
}
  \item{dist.method}{
String, the method used to calculate the distance between each vertex pair.
}
  \item{vertex.attr}{
Character vector, the name of the vertex attributes under which the vertex weights to be tested are stored. The vector can contain one or more elements.
}
  \item{edge.attr}{
String, the name of the edge attribute to be used as distances along the edges.
}
\item{correct.factor}{
Numeric value, if the network contains unconnected vertices, then the distance between these vertices is set as the maximum distance between the connected vertices multiplied by \code{correct.factor}.
}
  \item{nsteps}{
Integer value, the number of bins into which vertex pairs are placed.
}
  \item{B}{
Symmetric numerical matrix. A precomputed distance bin matrix for \code{g} output by the \code{BinGraph} function. If \code{NULL}, then B is computed within the \code{Knode} function. 
}
  \item{verbose}{
Logical, if \code{TRUE} messages about the progress of the function are displayed. 
}
}
\details{
Using the inner sum of the \code{Knet} equation, it becomes possible to prioritise vertices by how well they are connected, or associated, with high-weight vertices. The inner sum of the Knet equation is

\deqn{K^{node}_i[s]=\frac{2}{p} \sum_j (p_j - \bar{p}) I(d^g(i,j)<=s)}{K^node_i[s] = 2/p * sum_j(p_j - bar{p}) (dg(i,j)<=s)}

where \eqn{p_j} is the weight of vertex \eqn{j}, \eqn{\bar{p}} is the mean vertex weight across all vertices, and \eqn{I(dg[i,j]<=s)} is an identity function, equaling 1 if vertex \eqn{i} and vertex \eqn{j} are within distance \eqn{s} and \eqn{0} otherwise. 

If the name of each vertex is stored within a vertex attribute called \code{name}, then the returned scores are labelled with these names.

Vertex weights should be greater than or equal to 0, or equal to NA if the weight is missing. The Knode statistic is still computed for vertices with missing weights. 

If an edge attribute with this name is not found, then each edge is assumed to have a distance of 1. Smaller edge distances denote stronger interactions between vertex pairs
}
\value{
A sorted named numerical vector of \code{Knode} AUKs for each vertex. 

If one vertex attribute is input, then the \code{Knode} AUKs are calculated and a single numerical vector is returned. If more than one vertex attribute is input, then a list of vectors, one for each set of vertex weights, is returned.
}
\references{
Cornish, A.J. and Markowetz, F. (2014) \emph{SANTA: Quantifying the Functional Content of Molecular Networks.}. PLOS Computational Biology. 10:9, e1003808.
}
\author{
Alex J. Cornish \email{a.cornish12@imperial.ac.uk} and Florian Markowetz
}
\seealso{
\code{\link[SANTA:Knet]{Knet}}
}
\examples{
# create a network with a single cluster of high-weight vertices
# rank all vertices by their strength of association with the high-weight vertices
g1 <- erdos.renyi.game(15, p.or.m=0.3, directed=FALSE)
g1 <- SpreadHits(g1, h=3, lambda=10)
Knode(g1, vertex.attr="hits")
plot(g1)
}

\name{Compactness}
\alias{Compactness}
\title{
Measure the strength of association using compactness scores
}
\description{
The compactness score of set of hits on a network is the mean distance between each pair of hits. By comparing the observed compactness score to the scores of permuted hit sets, it is possible to compute the significance of the strength of association between the phenotype and the network. This method is not as effective as the \code{Knet} function and is included only for comparison.
}
\usage{
Compactness(g, nperm=100, dist.method=c("shortest.paths", "diffusion", "mfpt"), 
vertex.attr="pheno", edge.attr="distance", correct.factor=1, D=NULL, 
verbose=TRUE)
}
\arguments{
  \item{g}{
\code{igraph} object, the network to work on. 
}
  \item{nperm}{
Integer value, the number of permutations to be completed.
}
  \item{dist.method}{
String, the method used to compute the distance between each pair of hits on the network.
}
  \item{vertex.attr}{
Character vector, the name of the vertex attributes under which the hits to be tested are stored. The vector can contain one or more vertex attributes.
}
  \item{edge.attr}{
String, the name of the edge attribute to be used as distances along the edges. If an edge attribute with this name is not found, then each edge is assumed to have a distance of \code{1}. Smaller edge distances denote stronger interactions between vertex pairs.
}
\item{correct.factor}{
Numeric value. If the network contains unconnected vertices, then the distance between these vertices is set as the maximum distance between the connected vertices multiplied by \code{correct.factor}.
}
  \item{D}{
Symmetrical numerical matrix. A precomputed distance matrix for \code{g} output by the \code{DistGraph} function. If \code{NULL}, then \code{D} is computed by the \code{Compactness} function. 
}
  \item{verbose}{
Logical, if \code{TRUE} messages about the progress of the function are displayed. 
}
}
\details{
The compactness score is used by the PathExpand tool by Glaab et al. (2010). It is a measure of the mean distance between a set of genes in a network. By comparing the compactness score of an observed set of hits to sets of permuted hits, it is possible to produce a p-value describing the strength of association between the gene set and the network. This is not some done within the original paper by Glaab et al. (2010). The function is much like the \code{Knet} function, albeit not as effective.  

The compactness score \code{C} is defined as the mean shortest path distance between pairs of vertices in a set \code{P} on network \code{g}. 

\deqn{C(P) = \frac{2  \sum_{i,j \in P; i < j} d^g(i,j)}{|P| * (|P| - 1)}}

The compactness score is only included within the \code{SANTA} package to allow for comparisons to be made. Unlike the \code{Knet} function, it cannot be applied to continuous distributions of vertex weights. It can also result in biases if there is large variability in density across the network. 

The weight of a vertex should be 1 if it is a hit, 0 if it is not a hit or \code{NA} if the information is missing. Vertices with missing weights are still included within the network but are excluded from the permuted sets.
}
\value{
If one vertex attribute is input, \code{Compactness} is run on the single set of vertex weights and a list containing the statistics below is returned. If more than one vertex attribute is input, then \code{Compactness} is run on each set of vertex weights and a list containing an element for each vertex attribute is returned. Each element contains a sub-list containing the statistics below for the relevant vertex attribute.
\item{score.obs}{Observed compactness score}
\item{score.perm}{Permuted compactness scores. \code{NA} if no permutations are completed.}
\item{pval}{p-value, computed from a z-score derived from the observed and permuted compactness scores. \code{NA} if no permutations are completed.}
}
\references{
Cornish, A.J. and Markowetz, F. (2014) \emph{SANTA: Quantifying the Functional Content of Molecular Networks.}. PLOS Computational Biology. 10:9, e1003808.

Glaab, E., Baudot A., Krasnogor N. and Valencia A. (2010). \emph{Extending pathways and processes using molecular interaction networks to analyse cancer genome data}. BMC Bioinformatics. 11(1): 597:607.
}
\author{
Alex J. Cornish \email{a.cornish12@imperial.ac.uk} 
}

\name{DistGraph}
\alias{DistGraph}
\title{
Compute the vertex pair distance matrix for a graph
}
\description{
Compute the distances between pairs of vertices in a graph, using a shortest path, diffusion kernel, or mean first-passage time-based measure. 
}
\usage{
DistGraph(g, v=V(g), edge.attr=NULL, dist.method=c("shortest.paths", "diffusion", "mfpt"), 
correct.inf=TRUE, correct.factor=1, verbose=TRUE)
}
\arguments{
  \item{g}{
\code{igraph} object, the graph on which to work.
}
  \item{v}{
\code{igraph} object or numeric vector, the vertices from which each distance is calculated. 
}
  \item{edge.attr}{
String, the name of the edge attribute to be used as distances along the edges. If \code{NULL}, then each edge is assumed to have a distance of 1. Smaller edge distances denote stronger interactions between vertex pairs.
}
  \item{dist.method}{
String, the method used to compute the distance between each vertex pair.
}
  \item{correct.inf}{
Logical, if \code{TRUE} then infinite vertex pair distances are replaced with distances equal to the maximum distance measured across the network, multiplied by \code{correct.factor}.
}
\item{correct.factor}{
Numeric value, if the graph contains unconnected vertices, then the distance between these vertices is set as the maximum distance between the connected vertices multiplied by \code{correct.factor}.
}
  \item{verbose}{
Logical, if \code{TRUE} messages about the progress of the function are displayed. 
}
}
\details{
This function computes a distance matrix for a graph. Different methods can be used to calculate the distance between each pair of vertices. If a set of vertices is specified, a smaller distance matrix containing only vertices corresponding to the vertices is returned.

Descriptions of how the shortest paths (\code{shortest.paths}), diffusion kernel-based (\code{GraphDiffusion}) and mean first-passage time (\code{GraphMFPT}) distance measures work are given in their respective function descriptions.
}
\value{
Numeric matrix, containing the distances between vertex pairs. 
}
\references{
Kondor, R.I. and Lafferty, J. (2002). \emph{Diffusion Kernels on Graph and Other Discrete Structures}. Proc. Intl. Conf. Machine Learning. 

White, S. and Smyth, P. (2003). \emph{Algorithms for Estimating Relative Importance in Networks}. Technical Report UCI-ICS 04-25.
}
\author{
Alex J. Cornish \email{a.cornish12@imperial.ac.uk}
}
\seealso{
\code{\link[igraph]{shortest.paths}},
\code{\link[SANTA:GraphDiffusion]{GraphDiffusion}},
\code{\link[SANTA:GraphMFPT]{GraphMFPT}}
}
\examples{
# create a graph and compute the distance matrix using the shortest paths measure
g1 <- barabasi.game(6, directed=FALSE)
DistGraph(g1, dist.method="shortest.paths")
plot(g1, layout=layout.fruchterman.reingold)

# create a graph, assign edge distances and compute the distance matrix using the 
# diffusion kernel-based measure
g2 <- erdos.renyi.game(6, p.or.m=0.5, directed=FALSE)
g2 <- set.edge.attribute(g2, name="distance", value=runif(ecount(g2)))
DistGraph(g2, dist.method="diffusion", edge.attr="distance")
plot(g2, layout=layout.fruchterman.reingold)
}

\name{SpreadHits}
\alias{SpreadHits}
\title{
Spread hits across a network in clusters
}
\description{
Spread hits across a network in one or more clusters using an exponential probability distribution related to the distance of each vertex from a seed vertex. 
}
\usage{
SpreadHits(g, h, clusters=1, distance.cutoff=3, lambda=1, 
dist.method=c("shortest.paths", "diffusion", "mfpt"), 
edge.attr=NULL, hit.color="red", D=NULL, attempts=1000, verbose=TRUE)
}
\arguments{
  \item{g}{
\code{igraph} object, the network to work on.
}
  \item{h}{
Integer value, the number of hits in each cluster.
}
  \item{clusters}{
Integer value, the number of clusters to add the network. 
}
  \item{distance.cutoff}{
Integer value, the minimum distance between the seed vertex of each cluster. 
}
  \item{lambda}{
Numeric value, the strength of hit clustering. If \code{lambda == 0}, then the hits are randomly distributed. If \code{lambda > 0}, then the hits are clustered. The greater the value of \code{lambda}, the greater the strength of the hit clustering. 
}
  \item{dist.method}{
String, the method used to compute the distance between each vertex and the seed vertex. 
}
  \item{edge.attr}{
String, the name of the edge attribute to be used as distances along the edges. If \code{NULL}, then each edge is assumed to have a distance of \code{1}. Smaller edge distances denote stronger interactions between vertex pairs.
}
  \item{hit.color}{
String, the colour of the hits if the network is plotted.
}
  \item{D}{
Pre-computed numeric distance matrix. If \code{NULL}, then \code{D} is computed within the function.
}
  \item{attempts}{
Integer value, the number of attempts made at each stage of the placement algorithm before the function terminates. 
}
  \item{verbose}{
Logical, if \code{TRUE} messages about the progress of the function are displayed. 
}
}
\details{
\code{SpreadHits} can be used to add hits to a network without hits, or replace hits on a network with hits. The probability of a vertex being labelled as a hit depends on its distance from a randomly chosen starting vertex. The value of \code{lambda} denotes the shape of the probability distribution used to spread the hits. The greater the value of \code{lambda}, the greater the strength of hit clustering. The probability of vertex i being a hit is proportional to

\deqn{P[i] \sim lambda^{-d[start,i]}}

where \eqn{d[start,i]} is the distance between the start vertex and vertex i.  

Hits can be added in one or more clusters. If multiple clusters are being added, then the function first identifies an equal number of seed vertices at least \code{distance.cutoff} distance apart in the network. If the function cannot identify vertices that satisfy this condition within the specified number of attempts, then it returns \code{NULL}. If the function is able to find suitable seed vertices, then it proceeds to label vertices positioned around each seed vertex as hits, using a probability function shaped by \code{lambda}.

Whether a vertex is a hit or a miss is given in the vertex attribute \code{hits}. If the vertex is a hit, then it has the score 1. If it is a miss, then it has the score 0. A color can also be chosen to highlight the hits when the network is plotted. Misses are automatically coloured grey. 
}
\value{
A modified version of the input \code{igraph} object. Whether a vertex is a hit or miss is given under the vertex attribute \code{hits}. 
}
\author{
Alex J. Cornish \email{a.cornish12@imperial.ac.uk}
}
\examples{
# create a network and add 1 cluster of 3 hits
g1 <- erdos.renyi.game(30, p.or.m=0.1, directed=FALSE)
g1 <- SpreadHits(g1, h=3, clusters=1, lambda=10)
plot(g1, layout=layout.fruchterman.reingold)

# create a network and add 2 clusters of 3 hits
g2 <- erdos.renyi.game(30, p.or.m=0.1, directed=FALSE)
g2 <- SpreadHits(g2, h=3, clusters=3, distance.cutoff=2, lambda=0)
plot(g2, layout=layout.fruchterman.reingold)
}

\name{plot.Knet}
\alias{plot.Knet}
\title{
Plot the results of the \code{Knet} function
}
\description{
Plot the observed Knet curve against the quantiles of the permuted Knet curves and the observed AUK against the permuted AUKs. 
}
\usage{
\S3method{plot}{Knet}(x, sequential=FALSE, \dots)
}
\arguments{
  \item{x}{
Results from the Knet function.
}
  \item{sequential}{
Logical, if \code{TRUE} then the plots are sequential. Otherwise, the two plots are plotted alongside each other.
}
  \item{\dots}{
Additional arguments to be passed to \code{plot}.  	
}
}
\details{
If the high-weight vertices are clustered, then the observed Knet curve and AUK will be high relative to the permuted Knet curves and AUKs. The greater the strength of clustering, the greater the difference between the observed and permuted statistics. If the strength of clustering is low, then the observed and permuted curves and AUKs will likely overlap.

The first plot displays the the observed curve in red and the quantiles of the permuted curves in yellow. The quantile boundaries are displayed as grey lines. These boundaries are specified in the Knet function. The second plot displays the observed AUK as a red line and the distribution of permuted AUKs in grey. 
}
\value{
Plots described in details section.
}
\references{
Cornish, A.J. and Markowetz, F. (2014) \emph{SANTA: Quantifying the Functional Content of Molecular Networks.}. PLOS Computational Biology. 10:9, e1003808.
}
\author{
Alex J. Cornish \email{a.cornish12@imperial.ac.uk} and Florian Markowetz
}
\seealso{
\code{\link[SANTA:Knet]{Knet}}
}
\examples{
# plot results with hit clustering
g.clustered <- barabasi.game(100, directed=FALSE)
g.clustered <- SpreadHits(g.clustered, h=10, lambda=10)    
res.clustered <- Knet(g.clustered, nperm=10, vertex.attr="hits")
res.clustered$pval
plot.Knet(res.clustered)

# plot results without hit clustering
g.unclustered <- barabasi.game(100, directed=FALSE)
g.unclustered <- SpreadHits(g.unclustered, h=10, lambda=0)
res.unclustered <- Knet(g.unclustered, nperm=10, vertex.attr="hits")
res.unclustered$pval
plot.Knet(res.unclustered)
}

\name{GraphDiffusion}
\alias{GraphDiffusion}
\title{
Compute diffusion kernel-based distance matrix
}
\description{
Using a diffusion kernel-based algorithm, compute the distance between vertex pairs in an undirected network, with or without edge weights. This algorithm provides an alternative to the \code{shortest.paths} and \code{mfpt} measures of vertex pair distance.
}
\usage{
GraphDiffusion(g, v=V(g), edge.attr.weight=NULL, beta=1, correct.neg=TRUE)
}
\arguments{
  \item{g}{
\code{igraph} object, the network to work on. 
}
  \item{v}{
\code{igraph} object or numeric vector, the vertices from which each distance is calculated. 
}
  \item{edge.attr.weight}{
String, the name of the edge attribute to be used as weights along the edges. Greater weights indicate a stronger interaction between the two genes (this is the opposite to edge distances, where smaller distances indicate stronger interactions). If \code{NULL}, then each edge is assumed to have a weight of \code{1}.  
}
  \item{beta}{
Numeric value, the probability that the diffusion process will take an edge emanating from a vertex. 
}
  \item{correct.neg}{
Logical, if \code{TRUE} then negative edge distances are set to \code{0}. 
}
}
\details{
Diffusion across a network follows a process similar to a random walk. This provides a method of measuring the distance between vertex pairs that does not simply take into account a single path (like the \code{shortest.paths} algorithm) but instead incorporates multiple paths. This function uses a diffusion kernel-based approach to compute distances. The algorithm implemented is detailed in the referenced paper. 

The distance from vertex A to vertex A is always \code{0}.
}
\value{
Numeric matrix, containing the diffusion kernel-based vertex pair distances between each vertex in \code{v} and every vertex in \code{g}.
}
\references{
Kondor, R.I. and Lafferty, J. (2002). \emph{Diffusion Kernels on Graph and Other Discrete Structures}. Proc. Intl. Conf. Machine Learning. 
}
\author{
Alex J. Cornish \email{a.cornish12@imperial.ac.uk}
}
\seealso{
\code{\link[SANTA:GraphMFPT]{GraphMFPT}},
\code{\link[igraph]{shortest.paths}}
}
\examples{
# create a network and computes the diffusion kernel-derived vertex pair distance matrix
g <- barabasi.game(6, directed=FALSE)
GraphDiffusion(g)
plot(g, layout=layout.fruchterman.reingold)
}

\name{CreateGrid}
\alias{CreateGrid}
\title{
Generate a grid-like network
}
\description{
Generate a network with a grid-like arrangement of edges.
}
\usage{
CreateGrid(n=100)
}
\arguments{
  \item{n}{
Integer value, the number of vertices to be included. 
}
}
\details{
This is a simple algorithm that creates a grid-like network. Vertices are arranged in the largest square lattice possible. Vertices not included within this square are added as an additional row. Vertices are connected by edges to their closest neighbours.
}
\value{
\code{igraph} object.
}
\author{
Alex J. Cornish \email{a.cornish12@imperial.ac.uk}
}
\examples{
# generate and plot a grid-like network containing 100 vertices. 
g <- CreateGrid(n = 100)
plot(g, layout=layout.fruchterman.reingold)
}
